
##     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
##     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

##     This library is free software; you can redistribute it and/or modify it
##     under the terms of the GNU Lesser General Public License as published by the
##     Free Software Foundation; either version 2.1 of the License, or (at your
##     option) any later version.

##     This library is distributed in the hope that it will be useful, but WITHOUT
##     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
##     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
##     for more details.

##     You should have received a copy of the GNU Lesser General Public License
##     along with this library; if not, write to the Free Software Foundation,
##     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
##     http://www.gnu.org/licenses/lgpl.html

##     Author Contact Information:
##     Jim Sethna      
##     sethna@lassp.cornell.edu  
##     117 Clark Hall
##     Cornell University
##     Ithaca, NY 14850

import sys
# Unix
sys.path.append('../../src/MD/SWIGBuild/i686/obj3D')
# Windows
sys.path.append('../../src/MD/SWIGBuild/')

from MD3D import *
from Numeric import *

def minimize1D(function, initGuess, increment, accuracy):
    """Minimize a function of one variable and return the absissa"""
    dx = increment
    x = initGuess
    fold = function(x)
    while abs(dx) > accuracy:
        fnow = fold
        while fold >= fnow:
            x = x + dx
            fold, fnow = fnow, function(x)
        fold = fnow
        dx = -dx/10.
    return x

# here is the potential we want to study.
# See DigitalMaterial/src/MD/Potentials for other implemented potentials
#potential = EMTPotential("Cu")
#potential = StillingerWeberPotential()
#potential = SiliconEDIPPotential()
potential = CutLennardJonesPotential()
cutoff = potential.GetCutoffDistance()

# what kind of lattice is it?
lattice = FCCLattice(1.)

#lattice = DiamondLattice(1.);

# this has to be a lattice having the method SetLatticeConstant

plot = PlotAtomsObserver(0,0,8,8,8)



def atomicEnergy(latticeConstant):
    lattice.SetLatticeConstant(latticeConstant)

    nl = SimpleNeighborLocator(cutoff)
    bc = SimplePeriodicBoundaryConditions()
    lengths = array([4*latticeConstant,4*latticeConstant,4*latticeConstant])
    bc.SetLength(lengths)
    
    atoms = DynamicListOfAtoms()
    # because the number of atoms is quite small, we can use a simpleNeighborLocator

    atoms.SetBoundaryConditions(bc)
    atoms.SetNeighborLocator(nl)
    rect = RectangularClusterInitializer(lengths, lattice)
    center = array([lengths[0]/2,lengths[1]/2.,lengths[2]/2.])
    rect.SetCenter(center)
    rect.Create(atoms)
    atoms.AddObserver(plot)
    atoms.Notify()
    # this is the atoms which are close to the center
    #resNl = nl.NeighborsFromCenter([0.,0.,0.])  
    #center is the index of the central atom
    #center = resNl[1][argmin(resNl[3])]
    energy = potential.CalculateEnergy(atoms)/atoms.GetNumber()
    print energy
    return energy

# find the lattice spacing that minimizes the energy
if __name__ == '__main__':
    #latticeConstant = minimize1D(atomicEnergy, 10.0, -0.5, 1.0e-6)
    #latticeConstant = minimize1D(atomicEnergy, 5.4, +0.5, 1.0e-6)
    latticeConstant = minimize1D(atomicEnergy, 1.2, +0.1, 1.0e-10)
    print 'the latticeConstant that minimizes the energy in a', `lattice`, ' is', \
        latticeConstant , ' \n and the atomic energy is ', \
        atomicEnergy(latticeConstant)

