
##     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
##     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

##     This library is free software; you can redistribute it and/or modify it
##     under the terms of the GNU Lesser General Public License as published by the
##     Free Software Foundation; either version 2.1 of the License, or (at your
##     option) any later version.

##     This library is distributed in the hope that it will be useful, but WITHOUT
##     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
##     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
##     for more details.

##     You should have received a copy of the GNU Lesser General Public License
##     along with this library; if not, write to the Free Software Foundation,
##     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
##     http://www.gnu.org/licenses/lgpl.html

##     Author Contact Information:
##     Jim Sethna      
##     sethna@lassp.cornell.edu  
##     117 Clark Hall
##     Cornell University
##     Ithaca, NY 14850


import sys
# Unix
sys.path.append('../../src/MD/SWIGBuild/i686/obj3D')
# Windows
sys.path.append('../../src/MD/SWIGBuild/')
import MD3D
from Numeric import *
from LinearAlgebra import * # for matrixmultiply

def fileWrite(file,array):
    nAtoms = len(array)
    for i in xrange(nAtoms):
        s = '%15.8f %15.8f %15.8f\n' %(array[i][0],array[i][1],array[i][2])
        file.write(s)

# we take data as a function of a particular component of strain. Data recorded is energy density, and all six components of stress as calculated by the potential itself--not necessarily correct in a multi-atom unit cell.

strainCpt = 'xy' # can be 'xx','yy','zz','xy','yx','yz','zy','xz','zx'



root2Inv = 1./sqrt(2.)
root3Inv = 1./sqrt(3.)

# running variables
material = "LJ" #"SW""EDIP"
plotting = 0
dt = 0.01
dtQM = 0.01
tol = 1.0e-12
tolCG = 1.0e-10
friction = 0.1
temperature = 0.5
stepsPerStress = 5
maxIterations = 1000

nCells = 4 # number of units cells along a side! actual number of cells will be the cube of this

startStrain = -0.001
endStrain = 0.001
strainStep = 0.00005

# geometrical variables

#note that for the Stillinger Weber potential, the unit of length is 0.20951nm, and the unit of energy is 3.4723x10^(-19) J.
# therefore the unit of stress is (energy/force^3) 3.7757x10^10 Pa



#bc = MD3D.SimplePeriodicBoundaryConditions()
bc = MD3D.SkewPeriodicBoundaryConditions()
if material == "EDIP":
    potential = MD3D.SiliconEDIPPotential()
elif material == "SW":
    potential = MD3D.StillingerWeberPotential(2)
elif material == "LJ":
    potential = MD3D.CutLennardJonesPotential()
    
if material == "EDIP" or material == "SW":
    # diamond lattice
    latticeConst = potential.GetLengthScale() * 4./sqrt(3.)
elif material == "LJ":
    # FCC lattice
    latticeConst = potential.GetLengthScale() * sqrt(2.)


qm = MD3D.QuickMinAtomsMover(potential,dtQM, tol)
qm.SetMaxIterations(maxIterations)
#qm.Verbose()

#colorMethod = MD3D.ForceColorMethod(potential)
colorMethod = MD3D.StressColorMethod(potential)
colorMethod.SetComponent(1)

if plotting:
    plot = MD3D.PlotAtomsObserver(0.,nCells*latticeConst,0.,nCells*latticeConst,8)
    #plot = MD3D.RasMolObserver(8,colorMethod)



if material == "EDIP" or material == "SW":
    lattice = MD3D.DiamondLattice(latticeConst)
elif material == "LJ":
    lattice = MD3D.FCCLattice(latticeConst)


undeformedSuperCell = nCells*latticeConst*identity(3)
size = array([undeformedSuperCell[0][0],undeformedSuperCell[1][1],undeformedSuperCell[2][2]])
center = []
for coord in xrange(3):
    center.append(size[coord]/2)

# open data file

if strainCpt == 'xx':
    strainFile = open('CxxData.dat','w')
elif strainCpt == 'yy':
    strainFile = open('CyyData.dat','w')
elif strainCpt == 'zz':
    strainFile = open('CzzData.dat','w')
elif strainCpt == 'yz' or strainCpt == 'zy':
    strainFile = open('CyzData.dat','w')
elif strainCpt == 'xz' or strainCpt == 'zx':
    strainFile = open('CxzData.dat','w')
elif strainCpt == 'xy' or strainCpt == 'yx':
    strainFile = open('CxyData.dat','w')

# start loop over strains
nStrains = (endStrain-startStrain)/strainStep + 1
print 'nStrains is ', nStrains
strain = startStrain

for idx in xrange(nStrains):
    print 'Strain: ', strain


    if strainCpt == 'xx':
        strainMatrix = array([[strain,0.,0.],[0.,0.,0.],[0.,0.,0.]])
    elif strainCpt == 'yy':
        strainMatrix = array([[0.,0.,0.],[0.,strain,0.],[0.,0.,0.]])
    elif strainCpt == 'zz':
        strainMatrix = array([[0.,0.,0.],[0.,0.,0.],[0.,0.,strain]])
    elif strainCpt == 'yz' or strainCpt == 'zy':
        strainMatrix = array([[0.,0.,0.],[0.,0.,strain],[0.,strain,0.]]) 
    elif strainCpt == 'xz' or strainCpt == 'zx':
        strainMatrix = array([[0.,0.,strain],[0.,0.,0.],[strain,0.,0.]])
    elif strainCpt == 'xy' or strainCpt == 'yx':
        strainMatrix = array([[0.,strain,0.],[strain,0.,0.],[0.,0.,0.]])

    deformationMatrix = identity(3) + strainMatrix
    superCellVectors = matrixmultiply(deformationMatrix, undeformedSuperCell)
                                      
    linear = MD3D.LinearPositionTransformer(strainMatrix)

    bc.SetSuperCellVectors(superCellVectors)

    #neighborLocator = MD3D.CellNeighborList(potential.GetCutoffDistance(), 0.05 * potential.GetCutoffDistance())
    neighborLocator = MD3D.SimpleNeighborLocator(potential.GetCutoffDistance())
        
    # make a rectangular region 
    atoms = MD3D.DynamicListOfAtoms() 

    rect = MD3D.RectangularClusterInitializer(size,lattice)
    rect.SetRectangleSize(size)
    rect.SetCenter(center)
    rect.Create(atoms)

     
    print 'number of atoms ', atoms.GetNumber()

    if plotting:
        atoms.AddObserver(plot)
    atoms.Notify()
    
    linear.Transform(atoms)
    atoms.Notify()
    
    #atoms.Notify()
    #file = open("testAtoms",'w')
    #fileWrite(file,atoms.GetCartesianPositions())
    #oldPos = atoms.GetCartesianPositions()[:]
    atoms.SetNeighborLocator(neighborLocator)
    atoms.SetBoundaryConditions(bc)


    qm.Move(atoms)
    print 'qm convergence ', qm.GetConvergence()
    print 'qm iterations ', qm.GetNIterations()
    atoms.Notify()





    print 'calculating stress tensor ... '
    totalStress = [0,0,0,0,0,0]
    for i in xrange(atoms.GetNumber()):
        stress = potential.CalculateStress(atoms,i)
        for j in xrange(6):
            totalStress[j] = totalStress[j] + stress[j]#MD3D.ptrvalue(stress,j)
    for j in xrange(6):
        totalStress[j] = totalStress[j]/atoms.GetNumber()

    print 'Stress (all) ', totalStress
    
    volume = (nCells * latticeConst)**3
    energy = potential.CalculateEnergy(atoms)
    energyDensity = energy/volume
    print 'Energy Density', energyDensity


    s='%15.8f\t%15.8f\t%15.8f\t%15.8f\t%15.8f\t%15.8f\t%15.8f\t%15.8f\n' %(strain,energyDensity,totalStress[0],totalStress[1],totalStress[2],totalStress[3],totalStress[4],totalStress[5])
    strainFile.write(s)

    # increment strain for next step
    strain = strain + strainStep

strainFile.close()
