// ASCIIReader.h: interface for the ASCIIReader class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef ASCIIREADER_H
#define ASCIIREADER_H

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif //SWIG


#include "DMFirstHeader.h"
#include <string>
#include <fstream>
#include "DigitalMaterial.h"
#include "DMReader.h"

class ASCIIReader: public DMReader
{
public:
	ASCIIReader(const std::string &filename = "data.txt");
	virtual ~ASCIIReader();
	void Open();
	void Close();
	std::string GetType(const std::string &name="root"); 
	bool Fill(Serializable *toFill, const std::string &name="root");
	Serializable *Get(const std::string &name="root");
	bool GetBool( const std::string& name );
	int GetInt(const std::string &name);
	double GetDouble(const std::string &name);
	std::string GetString(const std::string &name);
	int *GetIntArray(std::vector<int> &dims, const std::string &name);
	double *GetDoubleArray(std::vector<int> &dims, const std::string &name);
private:
	void Done();
	std::string offset;
	const int SIZE;
	char *buffer;
	std::ifstream in;
};

#endif // ASCIIREADER_H
