// BinaryReader.cpp: implementation of the BinaryReader class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


#include "DigitalMaterial.h"
#include "BinaryReader.h"
#include "DMException.h"
#include "Serializable.h"
#include "SerialFactory.h"
#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BinaryReader::BinaryReader(const char *buffer): DMReader("")
{
    this->buffer      = buffer;
    this->currentPos  = 0;
}

BinaryReader::~BinaryReader()
{
}

void BinaryReader::SetBuffer(const char *buffer)
{
    this->buffer = buffer;
    this->currentPos = 0;
}


std::string BinaryReader::GetType(const string &name)
{
    throw DMException("BinaryReader", "GetType", 
        "Types are not put/read in binary buffers");
}

Serializable *BinaryReader::Get(const string &name)
{
    Serializable *serializable = SerialFactory::Instance()->New(GetString(""));
    serializable->Load(this);
    return serializable;
}



bool BinaryReader::Fill(Serializable *toFill, const string &name)
{
    throw DMException("BinaryReader", "Fill", "Not Implemented yet");
}

bool BinaryReader::GetBool( const string& name )
{
    int tmp = GetInt("");
    return (tmp != 0);
}

int BinaryReader::GetInt(const string &name)
{
    const int size = sizeof(int);
    const int *result = reinterpret_cast<const int *> (buffer + currentPos);
    currentPos += size;
    return *result;
}

double BinaryReader::GetDouble(const string &name)
{
    const int size = sizeof(double);
    const double *result = reinterpret_cast<const double *> (buffer + currentPos);
    currentPos += size;
    return *result;
}

string BinaryReader::GetString(const string &name)
{
    string result = buffer+currentPos;
    currentPos += result.size() + 1;
    return result;
}

int *BinaryReader::GetIntArray(vector<int> &dims, const string &name)
{
    dims.clear();
    const int nDims = GetInt("");
    int tot = 1;
    for(int i=0; i<nDims; ++i)
    {
        int tmp = GetInt("");
        dims.push_back(tmp);
        tot *= tmp;
    }
    int *data = new int[tot];
    memcpy(data, buffer+currentPos, tot*sizeof(int));
    currentPos += tot*sizeof(int);
    return data;	
}

double *BinaryReader::GetDoubleArray(vector<int> &dims, const string &name)
{
    dims.clear();
    const int nDims = GetInt("");
    int tot = 1;
    for(int i=0; i<nDims; ++i)
    {
        int tmp = GetInt("");
        dims.push_back(tmp);
        tot *= tmp;
    }
    double *data = new double[tot];
    memcpy(data, buffer+currentPos, tot*sizeof(double));
    currentPos += tot*sizeof(double);
    return data;	
}
