// BinaryReader.h: interface for the BinaryReader class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef BINARYREADER_H
#define BINARYREADER_H

#include "DigitalMaterial.h"

#include "DMReader.h"

class BinaryReader : public DMReader  
{
public:
    BinaryReader(const char *buffer = 0);
    virtual ~BinaryReader();
    //! Not used
    void Open() {}
    //! Not used
    void Close() {}
    std::string GetType(const std::string &name="root") ; 
    virtual bool Fill(Serializable *toFill, const std::string &name="root");
    virtual Serializable *Get(const std::string &name="root") ;
    virtual bool GetBool( const std::string& name ) ;
    virtual int GetInt(const std::string &name) ;
    virtual double GetDouble(const std::string &name) ;
    virtual std::string GetString(const std::string &name);
    virtual int *GetIntArray(std::vector<int> &dims, const std::string &name);
    virtual double *GetDoubleArray(std::vector<int> &dims, const std::string &name);

    template <class Type> 
        void GetArray(std::vector<int> &dims, const std::string &name, Type *&ip)
    { throw DMException("DMReader", "GetArray<>", "This type not implemented yet");}
    void GetArray(std::vector<int> &dims, const std::string &name, int *&ip)
                { ip = GetIntArray(dims, name);}
    void GetArray(std::vector<int> &dims, const std::string &name, double *&xp)
                { xp = GetDoubleArray(dims, name);}

    void SetBuffer(const char *buffer);

protected:
    const char *buffer;
    int currentPos;
};

#endif // !defined BINARYREADER_H
