// DMFirstHeader.h
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _DM_FIRST_HEADER_H_
#define _DM_FIRST_HEADER_H_

/*! \file DMFirstHeader.h
	\brief Should be the first header included---before system headers.

	Because Win32 system header files actually apply to several different
	OS versions (Win 3.1, Win95, Win2k, WinME, WinCE,...)  constants
	identifying the target system need to be defined before the system headers
	are read.

  Win32 constants are defined in
  http://msdn.microsoft.com/library/psdk/sdkintro/guide_04qb.htm .

	*/

/*! \def WINVER
	\brief Rough specification of version of target windows system.

	Set to 0x0400 for 95 or NT4.0.  Set to 0x0500 for Win2k or Win98.
	*/

/*! \def _WIN32_WINNT
	\brief Specifies version of target windows NT system.

	Setting this rules out Win95 or Win98.
	*/

/*! \def HAVE_BUGSLAYER
	\brief Whether you use Bugslayer DLL and utilities on a Windows build.

	BugslayerUtil.dll is a debugging library written by John Robbins and
	presented in his book "Debugging Applications."  It gives you super-nice
	assert functions, trace macros, and SEH Error Handling.

	Note that this library also requires DbgHelp.DLL from the resource kit.
	*/

#ifdef _MSC_VER

// These two make it definitely Win2k
#ifdef WIN2K
#define WINVER 0x0500
#define _WIN32_WINNT 0x0500
#endif

// 98 or Win2k
#ifdef WIN982K
#define WINVER 0x0500
#endif

// NT4 only
#ifdef WINNT4
#define _WIN32_WINNT 0x0400
#define WINVER 0x0400
#endif

// 98 or NT
#ifdef WIN98NT
#define _WIN32_WINDOWS 0x0410
#define WINVER 0x0400
#endif

// This says it's 95 or NT4.0
#ifndef WINVER
#define WINVER 0x0400
#endif

#pragma warning(disable:4786) // turns off spurious Windows STL warnings

// Defining this constant tells the compiler to use the default
// version of the debug new.
// MS recommends we define _CRTDBG_MAP_ALLOC here before
// loading stdlib.h and crtdbg.h, but they made an error in the
// implementation which renders it useless.  It is supposed
// to map new() to a debug version which records the line 
// number of the call which leaked, but it doesn't work.

#include <stdlib.h>
#include <crtdbg.h>

// If you are using DirectDraw, you need to define your version
// of DirectDraw.  We happen to use version 7 for 2D grids.
// Change this as you need to.
#define   DIRECTDRAW_VERSION 0x0700


#endif // _MSC_VER

#endif //_DM_FIRST_HEADER_H_
