
//     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
//     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

//     This library is free software; you can redistribute it and/or modify it
//     under the terms of the GNU Lesser General Public License as published by the
//     Free Software Foundation; either version 2.1 of the License, or (at your
//     option) any later version.

//     This library is distributed in the hope that it will be useful, but WITHOUT
//     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
//     for more details.

//     You should have received a copy of the GNU Lesser General Public License
//     along with this library; if not, write to the Free Software Foundation,
//     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
//     http://www.gnu.org/licenses/lgpl.html

//     Author Contact Information:
//     Jim Sethna      
//     sethna@lassp.cornell.edu  
//     117 Clark Hall
//     Cornell University
//     Ithaca, NY 14850


// ANY is a special keyword representing any number in the typemap (it's like a template). 
// $dim0 represents the value of the first ANY, $dim1 the value of the 2nd, etc...

// we can include a list or a numPyArray of ints everywhere we need an array of int.

%typemap(in) int [ANY], const int[ANY]
{
	
	PyArrayObject *arrayObj = (PyArrayObject *) PyArray_ContiguousFromObject($input, PyArray_INT, 1, 1);
	if( arrayObj == 0)
	{
		PyErr_SetString(PyExc_TypeError,"expected list, tuple or array of ints");
		return 0;
	}
	int arrayLength = arrayObj->dimensions[0];
	if($1_dim0 != arrayLength)
	{
		PyErr_SetString(PyExc_TypeError,"List/tuple/array had wrong length");
		return 0;
	}

	$1 = new int[$1_dim0];
	for(int i = 0; i < $1_dim0; ++i)
		$1[i] = ((int*)arrayObj->data)[i];

	Py_DECREF(arrayObj);
}

# we can include a list or a numPyArray of ints everywhere we need an array of bool. 
%typemap(in) bool [ANY], const bool[ANY]
{
	PyArrayObject *arrayObj = (PyArrayObject *) PyArray_ContiguousFromObject($input, PyArray_INT, 1, 1);
	if( arrayObj == 0)
	{
		PyErr_SetString(PyExc_TypeError,"expected list, tuple or array of ints");
		return 0;
	}
	int arrayLength = arrayObj->dimensions[0];
	if($1_dim0 != arrayLength)
	{
		PyErr_SetString(PyExc_TypeError,"List/tuple/array had wrong length");
		return 0;
	}

	$1 = new bool[$1_dim0];
	for(int i = 0; i < $1_dim0; ++i)
		$1[i] = ((int*)arrayObj->data)[i] != 0;

	Py_DECREF(arrayObj);
}



%typemap(in) double[ANY], const double[ANY]
{
	PyArrayObject *arrayObj = (PyArrayObject *) PyArray_ContiguousFromObject($input, PyArray_DOUBLE, 1, 1);
	if( arrayObj == 0)
	{
		PyErr_SetString(PyExc_TypeError,"expected list, tuple or array of double-castables");
		return 0;
	}
	int arrayLength = arrayObj->dimensions[0];
	if($1_dim0 != arrayLength)
	{
		PyErr_SetString(PyExc_TypeError,"List/tuple/array had wrong length");
		return 0;
	}

	$1 = new double[$1_dim0];
	for(int i = 0; i < $1_dim0; ++i)
		$1[i] = ((double*)arrayObj->data)[i];

	Py_DECREF(arrayObj);
}


%typemap(arginit) int[ANY], const int[ANY], bool[ANY], const bool[ANY], double[ANY], const double[ANY], double[ANY][ANY], const double[ANY][ANY]  {
    $1 = 0;
}



%typemap(python,freearg) int[ANY], const int[ANY], bool[ANY], const bool[ANY], double[ANY], const double[ANY], double[ANY][ANY], const double[ANY][ANY]
{
	if($1) 
		delete [] $1;	
}

# we want to map a double[ANY][ANY] to a NumPy array
%typemap(python,in) double [ANY][ANY], const double[ANY][ANY]
{
	PyArrayObject *arrayObj = (PyArrayObject *) PyArray_ContiguousFromObject($input, PyArray_DOUBLE, 2, 2);
	if( arrayObj == 0)
	{
		PyErr_SetString(PyExc_TypeError,"expected list, tuple or array of double-castables");
		return 0;
	}
	int arrayLengths[2] = {arrayObj->dimensions[0],arrayObj->dimensions[1]};
	if($1_dim0 != arrayLengths[0] || $1_dim1 != arrayLengths[1])
	{
		PyErr_SetString(PyExc_TypeError,"List/tuple/array had wrong shape");
		return 0;
	}

	$1 = new double[$1_dim0][$1_dim1];
	for(int i = 0; i < $1_dim0; ++i)
		for(int j = 0; j < $1_dim1; ++j)
		$1[i][j] = ((double*)arrayObj->data)[i*$1_dim0 + j];

	Py_DECREF(arrayObj);
}
