// LocalProtocol.h: interface for the LocalProtocol class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef LOCALPROTOCOL_H
#define LOCALPROTOCOL_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "DMProtocol.h"
#include "BinaryReader.h"
#include "BinaryWriter.h"

#include "DMException.h"

class LocalProtocol : public DMProtocol
{
public:
    LocalProtocol() {}
    virtual ~LocalProtocol() {}
    
    int GetProcessorNumber() const {return 0;}
    int GetNumberOfProcessors() const {return 1;}
    
    void Broadcast(double *values, int size, int master) {}
    void Broadcast(int *values, int size, int master) {}
    void Broadcast(std::string &string, int master) {}
    
    void Barrier() {}
    
    std::vector<int> GatherValue(int number, int root) {return std::vector<int>(1,number);}
        
    void AllReduce(const double *in, double *out, int size, Operation op);
    void AllReduce(const int *in   , int    *out, int size, Operation op);
    
    Buffers *NewBuffers() {return new BinBuffers();}

    class BinBuffers : public DMProtocol::Buffers
    {
    public:
        BinBuffers();
        ~BinBuffers();
        void AllToAll();
        DMWriter *GetSendBuffer(int proc){return sendBuffer;}
        DMReader *GetRecvBuffer(int proc){return recvBuffer;}
    private:
        //! Initialize the private members
        void Initialize();
        BinaryWriter *sendBuffer;
        BinaryReader *recvBuffer;
    };
};



#endif // !defined(LOCALPROTOCOL_H)
