// Buffer.h: interface for the Buffer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef MPIRECVBUFFER_H
#define MPIRECVBUFFER_H

#include "DMReader.h"
#include "mpi.h"

//! A class that provides a buffer where MPI can write to and allows to unpack passed data.
/*! The member data consist of the MPI communicator, the buffer where data is written,
the size of the buffer (\a bufferSize) and \a currentPos: the shift between the beginning 
of the buffer and the next data to be read.

These is a weakness in the present design: the size of the buffer can be changed, but this 
is not done automatically if the buffer is too small to receive what it should.
This could be fixed if we agreed to send the size of the buffer prior to sending the data.
*/

class MPIRecvBuffer: public DMReader
{
public:
	MPIRecvBuffer(const MPI_Comm comm = 0);
	virtual ~MPIRecvBuffer();
	void SetComm(MPI_Comm comm) {this->comm = comm;}
	void Allocate(int bufferSize);
	//! Not used
	void Open() {}
	//! Not used
	void Close() {}
	//! Resets \a currentPos to 0
	void EmptyBuffer() {this->currentPos = 0;}
	std::string GetType(const std::string &name="root"); 
	bool Fill(Serializable *toFill, const std::string &name="root");
	Serializable *Get(const std::string &name="root");
	bool GetBool( const std::string& name );
	int GetInt(const std::string &name);
	double GetDouble(const std::string &name);
	std::string GetString(const std::string &name);
	int *GetIntArray(std::vector<int> &dims, const std::string &name);
	double *GetDoubleArray(std::vector<int> &dims, const std::string &name);
	void *GetBuffer() {return buffer;}
	int GetBufferSize() {return bufferSize;}
private:
	MPI_Comm comm;
	char *buffer;
	int bufferSize;
	int currentPos;
};

#endif // !defined MPIRECVBUFFER_H



	
