// Buffer.h: interface for the Buffer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef MPISENDBUFFER_H
#define MPISENDBUFFER_H


#include "BinaryWriter.h"
#include "mpi.h"

//! A class that provides a buffer where a user can dump data and that MPI can send.
/*! The member data consist of the MPI communicator, the buffer where data is written,
the size of the buffer (\a bufferSize) and \a currentSize.
*/

class MPISendBuffer : public BinaryWriter
{
public:
    MPISendBuffer(const MPI_Comm comm=0);
    virtual ~MPISendBuffer();
    void SetComm(MPI_Comm comm) {this->comm = comm;}
    void PutBool( bool b, const std::string& name );
    void PutInt(int n, const std::string &name);
    void PutDouble(double x, const std::string &name);
    void PutString(const std::string &str, const std::string &name);
    void PutIntArray(const int *data, const std::vector<int> &dims, const std::string &name);
    void PutDoubleArray(const double *data, const std::vector<int> &dims, const std::string &name);
    void *GetBuffer() {return buffer;}
    int GetSize() {return currentSize;}
private:
    MPI_Comm comm;
};


#endif // !defined MPISENDBUFFER_H




