// Rand.cpp implementation of the Rand class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DMFirstHeader.h"
#include <vector>
#include <algorithm>

#include "Rand.h"
#include "DMWriter.h"
#include "DMReader.h"
#include "SerialBuilder.h"
#include "DMLastHeader.h"

const std::string sType = std::string("Rand");
const SerialBuilder<Rand> *_creator = new 
	SerialBuilder<Rand>(sType);

const double Rand::cd=7654321.0/16777216.0;
const double Rand::cm=16777213.0/16777216.0;

Rand::Rand(int seed) {
	this->iOriginalSeed = seed;
	ranmarin(abs(seed%BIGPRIME));
	outputReady=false;
}

Rand::Rand() {
	this->iOriginalSeed = 1;
	ranmarin(1);
	outputReady=false;
}

void Rand::Save( DMWriter* writer ) const
{
	std::vector<int> shape;
	int iOutput;
	iOutput = (outputReady) ? 1 : 0;
	writer->PutInt(iOutput, "isOutputReady" );
	writer->PutInt( i97, "i97" );
	writer->PutInt( j97, "j97" );
	writer->PutDouble( c, "c" );
	writer->PutDouble( cd, "cd" );
	writer->PutDouble( cm, "cm" );
	shape.push_back(97);
	writer->PutDoubleArray( u, shape, "u97" );
}

void Rand::Load( DMReader* reader)
{
	std::vector<int> shape;
	double *dTemp;
	int iOutput = reader->GetInt( "isOutputReady" );
	outputReady = (iOutput == 1);
	i97 = reader->GetInt( "i97" );
	j97 = reader->GetInt( "j97" );
	c = reader->GetDouble( "c" );
	double trash = reader->GetDouble( "cd" );
	trash = reader->GetDouble( "cm" );
	dTemp = reader->GetDoubleArray( shape, "u97" );
	std::copy(dTemp,dTemp+97, (double*) u);
	delete[] dTemp;
}

const std::string &Rand::GetType() const
{
	return sType;
}
