/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Calculates the first derivative of the weight function theta
 */

#include "vectools.h"

/*
 * d_theta/d_r = (-slope) * theta * (1 - theta)
 *
 * Called with a = theta and b = -slope:
 *
 * r[] = b * (a[] - a[] * a[])
 */
void vec_diffweight(double *res, const double *a, double b, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b)
#endif
    
    int i, limit;
    double a0, a1, a2, a3;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	res[i] = b * (a0 - a0 * a0);
	res[i+1] = b * (a1 - a1 * a1);	
	res[i+2] = b * (a2 - a2 * a2);	
	res[i+3] = b * (a3 - a3 * a3);
    }
    for (; i < n; i++)
    {
	a0 = a[i];
	res[i] = b * (a0 - a0 * a0);
    }
}
