/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Vectorized r = a * b + c
 */

#include "vectools.h"

/* r[] = a[] * b + c */
void vec_mul_scal_add_scal(double *res, const double *a, const double b, const double c, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, c)
#endif
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	res[i] = a0 * b + c;
	res[i+1] = a1 * b + c;
	res[i+2] = a2 * b + c;
	res[i+3] = a3 * b + c;
	res[i+4] = a4 * b + c;
	res[i+5] = a5 * b + c;
	res[i+6] = a6 * b + c;
	res[i+7] = a7 * b + c;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	res[i] = a0 * b + c;
    }
}

/* r[] = a[] * b[offset[]] + c[offset[]] */
void vec_mul_indir_add_indir(double *res, const double *a, const double *b, const double *c,
			     const int *offset, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c)
#endif
    
    int i, limit;
    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	b0 = b[offset[i]];
	c0 = c[offset[i]];
	a1 = a[i+1];
	b1 = b[offset[i+1]];
	c1 = c[offset[i+1]];
	a2 = a[i+2];
	b2 = b[offset[i+2]];
	c2 = c[offset[i+2]];
	a3 = a[i+3];
	b3 = b[offset[i+3]];
	c3 = c[offset[i+3]];
	res[i] = a0 * b0 + c0;
	res[i+1] = a1 * b1 + c1;
	res[i+2] = a2 * b2 + c2;
	res[i+3] = a3 * b3 + c3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[offset[i]];
	c0 = c[offset[i]];
	res[i] = a0 * b0 + c0;
    }
}
