/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = r * a + b * c + d
 */

#include "vectools.h"

/*   res[] = res[] * a + b[] * c + d[]   */
void vec_self_mul_scal_add_mul_scal_add(double *res, double a, const double *b,
					double c, const double *d, int n)
{
#ifdef AIX
#pragma disjoint (*res, a, *b, c, *d)
#endif
    
    int i,limit;
    double b0, b1, b2, b3;
    double d0, d1, d2, d3;
    double s0, s1, s2, s3;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	s3 = res[i+3];
	b0 = b[i];
	b1 = b[i+1];
	b2 = b[i+2];
	b3 = b[i+3];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	d3 = d[i+3];
	/* The parentheses help the compiler to generate FMA instructions */
	res[i] = s0 * a + (b0 * c + d0);
	res[i+1] = s1 * a + (b1 * c + d1);
	res[i+2] = s2 * a + (b2 * c + d2);
	res[i+3] = s3 * a + (b3 * c + d3);
    }
    for (i = n - n % 4; i < n; i++)
    {
	s0 = res[i];
	b0 = b[i];
	d0 = d[i];
	res[i] = s0 * a + (b0 * c + d0);
    }
}

/*   res[][0..2] = res[][0..2] * a + b[][0..2] * c[offset[]] + d[][0..2]   */
void vec3_self_mul_scal_add_mul_indir_add(double (*res3)[3], double a,
					  const double (*b3)[3], const double *c,
					  const double (*d3)[3], const int *offset, int n)
{
    int i,limit;
    double *res, *b, *d;
    double b0, b1, b2;
    double d0, d1, d2;
    double s0, s1, s2;
    double cx;
#ifdef AIX
#pragma disjoint (*res3, a, *b3, *c, *d3)
#pragma disjoint (*res, a, *b, *c, *d)
#endif

    res = (double *) res3;
    b = (double *) b3;
    d = (double *) d3;
    
    limit = 3*n;
    for (i = 0; i < limit; i+=3)
    {
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	b0 = b[i];
	b1 = b[i+1];
	b2 = b[i+2];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	cx = c[*offset++];
	/* The parentheses help the compiler to generate FMA instructions */
	res[i] = s0 * a + (b0 * cx + d0);
	res[i+1] = s1 * a + (b1 * cx + d1);
	res[i+2] = s2 * a + (b2 * cx + d2);
    }
}

/*   res[][0..2] = res[][0..2] * a[offset[]] + b[][0..2] * c[offset[]]
     + d[][0..2]   */
void vec3_self_mul_indir_add_mul_indir_add(double (*res3)[3], const double *a,
					   const double (*b3)[3], const double *c,
					   const double (*d3)[3], const int *offset, int n)
{
    int i,limit;
    double *res, *b, *d;
    double b0, b1, b2;
    double d0, d1, d2;
    double s0, s1, s2;
    double ax, cx;
#ifdef AIX
#pragma disjoint (*res3, *a, *b3, *c, *d3)
#pragma disjoint (*res, *a, *b, *c, *d)
#endif

    res = (double *) res3;
    b = (double *) b3;
    d = (double *) d3;
    
    limit = 3*n;
    for (i = 0; i < limit; i+=3)
    {
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	b0 = b[i];
	b1 = b[i+1];
	b2 = b[i+2];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	ax = a[*offset];
	cx = c[*offset++];
	/* The parentheses help the compiler to generate FMA instructions */
	res[i] = s0 * ax + (b0 * cx + d0);
	res[i+1] = s1 * ax + (b1 * cx + d1);
	res[i+2] = s2 * ax + (b2 * cx + d2);
    }
}
