/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = a * b + c * b + d * e
 */

#include "vectools.h"

/* r[] = a[] * b[] + c[] * b[] + d * e[] */
void vec_av_times_bv_plus_cv_times_bv_plus_d_times_ev(double *res, const double *a,
						      const double *b, const double *c,
						      double d, const double *e,
						      int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c, d, *e)
#endif
    
    int i, limit;
    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;
    double e0, e1, e2, e3;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	b0 = b[i];
	b1 = b[i+1];
	b2 = b[i+2];
	b3 = b[i+3];
	c0 = c[i];
	c1 = c[i+1];
	c2 = c[i+2];
	c3 = c[i+3];
	e0 = e[i];
	e1 = e[i+1];
	e2 = e[i+2];
	e3 = e[i+3];
	res[i] = a0 * b0 + c0 * b0 + d * e0;
	res[i+1] = a1 * b1 + c1 * b1 + d * e1;
	res[i+2] = a2 * b2 + c2 * b2 + d * e2;
	res[i+3] = a3 * b3 + c3 * b3 + d * e3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[i];
	c0 = c[i];
	e0 = e[i];
	res[i] = a0 * b0 + c0 * b0 + d * e0;
    }
}
