// SteepestDescentMinimizer.h: interface for the SteepestDescentMinimizer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef STEEPESTDESCENTMINIMIZER_H
#define STEEPESTDESCENTMINIMIZER_H

#include "DMFirstHeader.h"
#include <math.h>
#include <string>
#include "DigitalMaterial.h"
#include "Minimizer.h"


//! Does steepest descent minimization,taking a fixed step each time. Does not do actual line minimization so does not make use of the ObjectiveFunction.



/**
 *
 *
 * \author nbailey 
 * \date 9/21/2000
 * \ingroup moleculardynamics
 */




class SteepestDescentMinimizer: public Minimizer  
{
public:
  SteepestDescentMinimizer(double tolerance, int maximumIterations,double stepParameter);
  virtual ~SteepestDescentMinimizer();
	double Minimize(double *parameters,Minimizable *minimizable);  
  double GetTolerance() const {return tolerance;}
  void SetTolerance(double tol) {tolerance = tol;}
  int GetNIterations() const {return nIterations;} // returns number of iterations taken during the last call to minimize
  double GetFinalGradNorm() const {return gradNorm;}
  int GetMaxIterations() const {return maximumIterations;}
  void SetMaxIterations(int maxIt) {maximumIterations = maxIt;}
  
  void SetStepParameter(double sp) {stepParameter = sp;}
  double GetStepParameter() const {return stepParameter;}
  void Verbose() {verbose = true;}
  void Quiet() {verbose = false;}
  
 protected:
  int maximumIterations;
  double tolerance;
  int nIterations;	
  double gradNorm;
  Minimizable *minimizable;
  bool verbose;
#ifndef SWIG
  
  double stepParameter;
    
  double *parameters;
#endif // SWIG
  
};



#endif // !defined(STEEPESTDESCENTMINIMIZER_H)
