// Thread.h interface for the Thread class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _THREAD_H_
#define _THREAD_H_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif

#include "ThreadImp.h"

//! The system-independent implementation of threads.
/**
 * This is the actual Thread class.  It varies in its system-dependent
 * implementation which it gets from an abstract factory.
 * See ThreadImp for details.
 * \author Drew Dolgert
 * \date 21 Feb 2000
 * \ingroup dmsystem
 * \sa ThreadImp, WinThread, UnixThread, Threadable, Runnable
 */
class Thread : public ThreadImp
{
public:
	Thread(Runnable *runnable);
	Thread(Threadable *runnable);
	virtual ~Thread() { delete _thread; }
	void SetTask(Runnable *runnable) { _thread->SetTask(runnable); }
	void SetTask(Threadable *threadable) { _thread->SetTask(threadable); }
	int Start() { return _thread->Start(); }
	int Stop() { return _thread->Stop(); }
	int Pause() { return _thread->Pause(); }
	int Resume() { return _thread->Resume(); }
	int Toggle() { return _thread->Toggle(); }
	// Interface to return thread.
	bool IsRunning() { return _thread->IsRunning(); }
private:
	ThreadImp* _thread;
};

#endif // _THREAD_H_
