// TrivialThread.h interface for the TrivialThread class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _TRIVIALTHREAD_
#define _TRIVIALTHREAD_ 1

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif

#include "Threadable.h" // includes Runnable
#include "ThreadImp.h"

//! The Trivial implementation of a Thread.
/**
 * It always return SUCCESS.  It doesn't keep track of any inner logic as though there
 * were a thread.
 * \author ajd27
 * \date 28 July 2000
 * \ingroup dmsystem
 */
class TrivialThread : public ThreadImp
{
	Threadable* threadable;
	Runnable* runnable;
public:
	TrivialThread() : threadable(0), runnable(0) {};
	virtual void SetTask(Runnable *thread) { this->runnable = thread; }
	virtual void SetTask(Threadable *thread) { this->threadable = thread; }
	virtual int Start() { return SUCCESS; }
	virtual int Stop() { return SUCCESS; }
	virtual int Pause() { return SUCCESS; }
	virtual int Resume() { return SUCCESS; }
	virtual int Toggle() { return SUCCESS; }
	// interface to simulation
	virtual bool IsRunning() { return true; }
	virtual ~TrivialThread() {}
};


#endif // _TRIVIALTHREAD_
