// WinRWLock.cpp implementation of the WinRWLock class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"
#include "WinRWLock.h"
#include "DMLastHeader.h"

// Code from Ruediger R. Asche on Microsoft Developer Network.
/* implemented by Drew Dolgert
 * This code sorely needs error handling.
 * In addition, it looks to me like the writer may not have a
 * lock until all readers have left the area.  It should be that
 * as soon as a writer has asked for a lock no more readers may
 * enter, and when the last current reader leaves then the writer
 * gets the lock.
 *
 * We need timings, comparing this with an empty implementation.
 * If it is slow, consider whether we want the ListOfAtoms to call
 * Notify() on every step or whether it should be every nSteps steps.
 * That would allow us to lock much less frequently because we report
 * much less frequently.
 *
 * This implementation is synchronous.  It would be easy and secure
 * to do an asynchronous implementation.  I would even prefer it because
 * it would cede control to the GUI faster and cede control to the sim
 * faster, so it is better all-around.
 */

bool WinRWLock::InitializeLock()
{
	hReaderEvent = ::CreateEvent(0,true,false,0);
	hMutexEvent = ::CreateEvent(0,false,true,0);
	hWriterMutex = ::CreateMutex(0,false,0);
	iCounter = -1;
	return true;
}

void WinRWLock::CleanupLock()
{
	::CloseHandle(hReaderEvent);
	::CloseHandle(hMutexEvent);
	::CloseHandle(hWriterMutex);
}

void WinRWLock::ReadLock()
{
	if (::InterlockedIncrement(&iCounter)==0) {
		::WaitForSingleObject(hMutexEvent,INFINITE);
		::SetEvent(hReaderEvent);
	}
	WaitForSingleObject(hReaderEvent,INFINITE);
}

void WinRWLock::WriteLock()
{
	::WaitForSingleObject(hWriterMutex,INFINITE);
	::WaitForSingleObject(hMutexEvent,INFINITE);
}

void WinRWLock::ReadRelease()
{
	if (::InterlockedDecrement(&iCounter) < 0) {
		::ResetEvent(hReaderEvent);
		::SetEvent(hMutexEvent);
	}
}

void WinRWLock::WriteRelease()
{
	::SetEvent(hMutexEvent);
	::ReleaseMutex(hWriterMutex);
}
