// WinRWLock.h interface for the WinRWLock class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _WINRWLOCK_H_
#define _WINRWLOCK_H_

#include "DMFirstHeader.h"
#include <windows.h>
#include <process.h>
#include "DigitalMaterial.h"
#include "RWLockImp.h"

//! Read-write lock for Windows.
/**
 * This is the Windows implementation of read-write locks.
 * See RWLock for details on use.
 * \author Drew Dolgert
 * \date March 2000
 * \ingroup dmsystem
 */
class WinRWLock : public RWLockImp
{
	long iCounter; //! Number of threads reading.
	HANDLE hReaderEvent; //! Event readers wait on to enter.
	HANDLE hMutexEvent; //! Keeps readers and writers from mixing.
	HANDLE hWriterMutex; //! Keeps writers and writers from mixing.
public:
	virtual bool InitializeLock();
	virtual void CleanupLock();
	virtual void ReadLock();
	virtual void WriteLock();
	virtual void ReadRelease();
	virtual void WriteRelease();
};

#endif // _WINRWLOCK_H_
