/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
// MDMain.h : runs main loop (Console Application)
//
/////////////////////////////////////////////////////////////////////////////

#ifndef MDMAIN_H
#define MDMAIN_H

#include "DigitalMaterial.h"


#include "VerletAtomsMover.h"
#include "LangevinAtomsMover.h"
#include "CompositeLangevinAtomsMover.h"
#include "QuickMinAtomsMover.h"

#include "RectangularClusterInitializer.h"
#include "SphericalClusterInitializer.h"
#include "AtomicSurfaceInitializer.h"

#include "CutLennardJonesPotential.h"
#include "MultiCutLennardJonesPotential.h"
#include "StillingerWeberPotential.h"
#include "EMTPotential.h"

#include "NeighborList.h"
#include "CellNeighborLocator.h"
#include "CellNeighborList.h"
#include "SimpleNeighborLocator.h"
#include "CompositeNeighborLocator.h"
#include "BondBreakerNeighborLocator.h"

#include "DynamicListOfAtoms.h"
#if DIMENSION == 2
#include "2DLattices.h"
#else
#include "3DLattices.h"
#endif

#include "Constraint.h"
#include "FixedBody.h"

#include "SimplePeriodicBoundaryConditions.h"
#include "SmartPeriodicBoundaryConditions.h"

#include "ConjugateGradientMinimizer.h"
#include "SteepestDescentMinimizer.h"
#include "DislocationMaker.h"
#include "OverlapPruner.h"
#include "NotchMaker.h"

#ifdef LASSPTOOLS
#include "PlotAtomsObserver.h"
#endif //LASSPTOOLS

#ifdef WIN32
#include "WinProcessFactory.h"
#else
#include "UnixProcessFactory.h"
#endif //WIN32

//#include "Timer.h"

#include <fstream>
#include <string>
#include <strstream>
#include <stdexcept>


#include "blas.h"
#endif // !defined(MDMAIN_H)
