// AtomsInitializer.h: interface for the AtomsInitializer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef ATOMSINITIALIZER_H
#define ATOMSINITIALIZER_H


#include "DMFirstHeader.h"


class ListOfAtoms;

//! Abstract base class for classes which create atoms in certain configurations.
/*!	Abstract class Create needed by any AtomsInitializer.
	Under normal circumstances, all ListOfAtomses will be generated 
	by AtomsInitializers; see comments under ListOfAtoms.
	*/

class AtomsInitializer
{
public:
	//! This step actually creates the classes while others set up the cretaion.
	virtual void Create(ListOfAtoms *atoms)=0;
	//! Rough center (for graphics)
	virtual const double *GetCenter() const = 0;
	//! Rough size (for graphics)
	virtual double GetMaxSize() const = 0;	
};

#endif // !defined(ATOMSINITIALIZER_H)
