
//     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
//     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

//     This library is free software; you can redistribute it and/or modify it
//     under the terms of the GNU Lesser General Public License as published by the
//     Free Software Foundation; either version 2.1 of the License, or (at your
//     option) any later version.

//     This library is distributed in the hope that it will be useful, but WITHOUT
//     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
//     for more details.

//     You should have received a copy of the GNU Lesser General Public License
//     along with this library; if not, write to the Free Software Foundation,
//     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
//     http://www.gnu.org/licenses/lgpl.html

//     Author Contact Information:
//     Jim Sethna      
//     sethna@lassp.cornell.edu  
//     117 Clark Hall
//     Cornell University
//     Ithaca, NY 14850


%{
#include "AtomicSurfaceInitializer.h"
#include "ClusterInitializer.h"
#include "PolyClusterInitializer.h"
#include "RectangularClusterInitializer.h"
#include "SphericalClusterInitializer.h"
#include "CompositeClusterInitializer.h"
#include "CylindricalClusterInitializer.h"
#include "ParallelepipedClusterInitializer.h"
#include "EmptyInitializer.h"
%}

%nodefault AtomsInitializer;
class AtomsInitializer 
{
public:
	void Create(ListOfAtoms *atoms);
};

%nodefault ClusterInitializer;
class ClusterInitializer:public AtomsInitializer
{
public:
	void SetCenter(double clusterCenter[DIMENSION]);
	void SetBravaisLattice(BravaisLatticeWithBasis *bravais);
};

%include "AtomicSurfaceInitializer.h"
%include "RectangularClusterInitializer.h"
%include "SphericalClusterInitializer.h"

%include "CylindricalClusterInitializer.h"


class PolyClusterInitializer: public ClusterInitializer
{
public:
%extend
{
	PolyClusterInitializer(PyObject *pyCorners, BravaisLatticeWithBasis *lattice)
	{ // pyCorners are numPy arrays containing the corners.
		if(PyArray_Check(pyCorners))
		{
			PyArrayObject *o = (PyArrayObject *)pyCorners;
			int nCorners = o->dimensions[0];
			if (o->dimensions[1] != DIMENSION) std::cerr << " != DIMENSION !!!" << std::endl;
			if(o->descr->type_num == PyArray_DOUBLE)
			{
				double (*cornersData)[DIMENSION] = (double (*)[DIMENSION]) (o->data);
				ArrayDoubleVector corners(nCorners);
				for(int i=0; i<nCorners; ++i)
					for(int coord=0; coord<DIMENSION; ++coord)
						corners[i][coord] = cornersData[i][coord];
				PolyClusterInitializer *init = new PolyClusterInitializer(&corners,lattice);
				return init;
			}
			else
			{
				PyErr_SetString(PyExc_TypeError,"need a double NumPy Array");
				return NULL;
			}
		}
		else
		{
			PyErr_SetString(PyExc_TypeError,"not a NumPy Array");
			return NULL;
		}
	}
}
};

%include "CompositeClusterInitializer.h"
%include "ParallelepipedClusterInitializer.h"
%include "EmptyInitializer.h"