// CompositeClusterInitializer.cpp: implementation of the CompositeClusterInitializer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DMFirstHeader.h"
#include <stdlib.h>
#include <string>

#include "DigitalMaterial.h"
#include "CompositeClusterInitializer.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// here is the AND version

CompositeClusterInitializer::CompositeClusterInitializer(ClusterInitializer *c1, ClusterInitializer *c2, 
										 BravaisLatticeWithBasis *lattice,const char *method) : ClusterInitializer(lattice)
{
	this->c1 = c1;
	this->c2 = c2;
	c1->bravais = lattice;
	c2->bravais = lattice;
	if(!strcmp(method, "intersection"))
	  METHOD = CLUSTER_INTERSECTION;
	else if (!strcmp(method,"difference"))
	  METHOD = CLUSTER_DIFFERENCE;
	else if(!strcmp (method,"union"))
	  METHOD = CLUSTER_UNION;
}

CompositeClusterInitializer::~CompositeClusterInitializer()
{

}

int CompositeClusterInitializer::SetMaxSize()
{
  // Take the smaller one when doing intersection, otherwise take th ebigger one
	int tmp1 = c1->SetMaxSize(), tmp2 = c2->SetMaxSize();
	if(METHOD == CLUSTER_INTERSECTION)
	  {
	    if (tmp2<tmp1) tmp1 = tmp2;
	    for(int coord=0; coord<DIMENSION; ++coord)
	      {
		maxSize[coord] = c1->maxSize[coord];
		if(c2->maxSize[coord] < maxSize[coord]) maxSize[coord] = c2->maxSize[coord];
		minSize[coord] = c1->minSize[coord];
		if(c2->minSize[coord] > minSize[coord]) minSize[coord] = c2->minSize[coord];
	      }
	    return tmp2;
	  }
	else 
	  {
	    if (tmp2<tmp1) tmp2 = tmp1;
	    for(int coord=0; coord<DIMENSION; ++coord)
	      {
		maxSize[coord] = c1->maxSize[coord];
		if(c2->maxSize[coord] > maxSize[coord]) maxSize[coord] = c2->maxSize[coord];
		minSize[coord] = c1->minSize[coord];
		if(c2->minSize[coord] < minSize[coord]) minSize[coord] = c2->minSize[coord];
	      }
	    return tmp2;
	  }

}

bool CompositeClusterInitializer::Inside(double X[DIMENSION])
{
  switch(METHOD)
    {
    case CLUSTER_INTERSECTION:
      return c1->Inside(X) && c2->Inside(X);
    case CLUSTER_DIFFERENCE:
      return  c1->Inside(X) && !c2->Inside(X);
    case CLUSTER_UNION:
      return  c1->Inside(X) || c2->Inside(X);
    }
	ASSERT(false); // We should never get here.
	return false;
}
