// SphericalClusterInitializer.cpp: implementation of the SphericalClusterInitializer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"
#include "SphericalClusterInitializer.h"

#include "BravaisLatticeWithBasis.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SphericalClusterInitializer::SphericalClusterInitializer(double radius, BravaisLatticeWithBasis *bravais) : ClusterInitializer(bravais)
{	
	SetRadius(radius);
}

SphericalClusterInitializer::~SphericalClusterInitializer()
{

}

void SphericalClusterInitializer::SetRadius(double radius)
{
	this->radius = radius;
	// XXX JPS Added
	SetMaxSize();
}

int SphericalClusterInitializer::SetMaxSize()
{
	// Calculate maxsize[DIMENSION], the maximal value of each coordinates of the points inside a 
	// sphere of given radius. After some calculation one arrives at the simple following result.
	    int coord;
        double volumeOfSphere = 1.;
		double extRadius;  // extRadius is an extended radius, used to estimate how many cells are enclosed in the sphere
		extRadius = radius + DIMENSION*pow(bravais->GetVolumeOfCell(),1/(double)DIMENSION);
        double latticevect[DIMENSION], cartvect[DIMENSION], temp[DIMENSION];
        for(coord=0;coord<DIMENSION;coord++) temp[coord] = 0.;
	    for(int k=0; k<DIMENSION; k++) 
		{
	      for(coord=0;coord<DIMENSION;coord++) cartvect[coord] = (coord==k ? 1. : 0.);
	      bravais->LatticeCoordinatesFromCartesianCoordinates(latticevect,cartvect);
	      for(coord=0;coord<DIMENSION;coord++) temp[coord] += latticevect[coord]*latticevect[coord];
		}
	    for(coord=DIMENSION; coord>=2;coord-=2) volumeOfSphere *= extRadius*extRadius*2.*PI/(double)coord ;
	    if(coord==1) volumeOfSphere *= 2.*extRadius;
		for(coord=0;coord<DIMENSION;coord++)
		{
			maxSize[coord] = int(sqrt(temp[coord])*radius) + 1;
			minSize[coord] = -maxSize[coord];
		}
		double shift[DIMENSION];
		bravais->LatticeCoordinatesFromCartesianCoordinates(shift, clusterCenter);
		for(coord=0; coord<DIMENSION; coord++) 
		{
			maxSize[coord] += (int) shift[coord];
			minSize[coord] += (int) shift[coord];
		}
		return int(volumeOfSphere*bravais->GetNAtomsPerCell()/bravais->GetVolumeOfCell());
}

bool SphericalClusterInitializer::Inside(double X[DIMENSION])
{
	double r2=0, tmp;
	for (int coord=0; coord<DIMENSION; coord++)
	{
	      tmp = X[coord] - clusterCenter[coord];
	      r2 += tmp*tmp;
	}
	return (r2 < radius*radius);
}
