// SphericalClusterInitializer.h: interface for the SphericalClusterInitializer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SPHERICALCLUSTERINITIALIZER_H
#define SPHERICALCLUSTERINITIALIZER_H

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // SWIG

// Implementation of the ClusterInitializer factory method
// Will generate a circular or spherical cluster of atoms, depending on dimension
// One should set the radius and perhaps the center of the cluster before Create()

#include "ClusterInitializer.h"

class SphericalClusterInitializer : public ClusterInitializer
{
public:
  SphericalClusterInitializer(double radius, BravaisLatticeWithBasis *bravais);
  virtual ~SphericalClusterInitializer();
  void SetRadius(double radius); 
  double GetRadius() const {return radius;}
  

protected:
	double radius;
	int SetMaxSize();
	bool Inside(double X[DIMENSION]);
};

#endif // !defined(SPHERICALCLUSTERINITIALIZER_H)
