// AtomsMover.h: interface for the AtomsMover class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef ATOMSMOVER_H
#define ATOMSMOVER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "Serializable.h"
#include "AttributeError.h"
#include "Potential.h"

// Strategy design pattern: many different Movers act interchangibly


class ListOfAtoms;

#include "DMArray.h"

class AtomsMover : public Serializable
{
public:
	AtomsMover() {}
	virtual ~AtomsMover() {}
	virtual void Move(ListOfAtoms *atoms)=0;
	void SetPotential(Potential *potential) 
	{
		this->potential = potential;
	}
	// Time step only works for some AtomsMovers
	virtual double GetDt() const {throw AttributeError("AtomsMover","GetDt");}
	virtual void SetDt(double dt) {throw AttributeError("AtomsMover","SetDt");}

	virtual double GetTime() const {throw AttributeError("AtomsMover","GetTime");}
	virtual void SetTime(double time=0.) {
		throw AttributeError("Potential","SetTime");
	}


	void Save(DMWriter *writer) const {}
	void Load(DMReader *reader) {}	

protected:
	Potential *potential;
	static const std::string className;
};


#endif // !defined(ATOMSMOVER_H)
