// ConstraintVerletAtomsMover.h: interface for the ConstraintVerletAtomsMover class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef CONSTRAINTVERLETATOMSMOVER_H
#define CONSTRAINTVERLETATOMSMOVER_H


#include "DigitalMaterial.h"
#include "AtomsMover.h"
#include "DMArray.h"
#include "Constraint.h"

class Potential;

class ExternalStress;


class ConstraintVerletAtomsMover : public AtomsMover
{
 public:
	ConstraintVerletAtomsMover(Potential *potential = 0,double dt = 1., double tolerance = 1.e-6);
	~ConstraintVerletAtomsMover();

	double GetTolerance() const {return tolerance;}
	void SetTolerance(double t) {tolerance = t;}
	int GetMaxSteps() const {return maxSteps;}
	void SetMaxSteps(int n) {maxSteps = n;}
	double GetDt() const {return dt;}
	void SetDt(double t) {dt = t;}

	void Move(ListOfAtoms *atoms);
   
	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}

	void SetMinimize() {minimize = true;}
	void UnSetMinimize() {minimize = false;}
	
	// Accessing "return" information
	double GetFinalTFS() const {return generalizedForceSq;}
	int GetNIterations() const {return nIterations;}
    double GetResidualForceSq() const {return residualForceSq;}


	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
		{
			return className;
		}
    
 protected:
	void CalculateGeneralizedForces(ListOfAtoms *atoms, Constraint *constraint,DMArray<double,DIMENSION> *forces);
    void IncrementGeneralizedVelocitiesFromForces(ListOfAtoms *atoms);
	void Allocate(int nDOFs);
	
	//! if verbose is set, print out status every \a STEPSBETWEENDISPLAY iterations
	static const int STEPSBETWEENDISPLAY;

	bool minimize;
	bool verbose;
	int nIterations;
	double dt;
	int nDOFs;
	double tolerance;
	int maxSteps;

    Potential *potential;

    double residualForceSq;
	double generalizedForceSq;

	double *generalizedCoordinates;
	double *generalizedVelocities;
	double *generalizedForces;

    double *kineticMatrix;
private:
	static const std::string className;
};



#endif // CONSTRAINTVERLETATOMSMOVER_H
