// HooverAtomsMover.h: interface for the HooverAtomsMover class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

// From "Time-reversible equilibrium and nonequilibrium isothermal-isobaric simulations
// with centered-difference Stoermer algorithms
// B.L. Holian, A.J. DeGroot, W.G. Hoover, and C.G. Hoover,
// Phys Rev A 41, 4552 - 4553 (1990)

#ifndef HOOVERATOMSMOVER_H
#define HOOVERATOMSMOVER_H

#include "DigitalMaterial.h"
#include "AtomsMover.h"
#include "Potential.h"

class HooverAtomsMover : public AtomsMover  
{
public:
	HooverAtomsMover(Potential *potential = 0, double temperature = 0.0, double dt=0.01, double tau = 1.0, int nSteps=1);
	virtual ~HooverAtomsMover() {}

	void Move(ListOfAtoms *atoms);

	// Time step for Verlet algorithm
	double GetDt() const {return dt;}
	void SetDt(double dt) {this->dt = dt;}

	// Target temperature
	double GetTemperature() const {return temperature;}
	void SetTemperature(double temperature) {this->temperature = temperature;}

	// Time-reversible friction coefficient
	double GetZeta() const {return zeta;}
	void SetZeta(double zeta) {this->zeta = zeta;}

	// Thermostat relaxation time
	double GetTau() const {return tau;}
	void SetTau(double tau) {this->tau = tau;}

	// Number of time-steps per call.
	int GetNSteps() const {return nSteps;}
	void SetNSteps(int nSteps) {this->nSteps = nSteps;}
	
	const std::string &GetType() const
		{
			return className;
		}
#ifndef SWIG
    void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	
#endif //SWIG
protected:
	void Allocate(int nAtoms, const DMArray<double, DIMENSION> *array);
	double dt;
	int nAtoms;		// Tells how large a force array is stored
	int nSteps;
	DMArray<double, DIMENSION> *forces;

	double zeta;	// Time-reversible friction coefficient
	double tau;		// Thermostat relaxation time
	double temperature; // Target temperature
	static const std::string className;
};

#endif // !defined(HOOVERATOMSMOVER_H)
