// QuickMinProjection.cpp: implementation of the QuickMinProjection class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "QuickMinProjection.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "SerialBuilder.h"
#include "Constraint.h"

#include "DMLastHeader.h"


using namespace std;


static SerialBuilder<QuickMinProjection> *_QMPcreator = 
    new SerialBuilder<QuickMinProjection>("QuickMinProjection");

const string QuickMinProjection::className = string("QuickMinProjection");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


void QuickMinProjection::AdjustVelocities(const DMArray<double, DIMENSION> *forces, const DMArray<double, DIMENSION> *velocities)
{
	// This works atom by atom.
	int coord;
	double vDotF, fSq, fSqInv;
	double *veloCurr, *forcesCurr, *tVeloCurr;

	DMArrayIterator<double, DIMENSION> forcesIter(forces), veloIter(velocities), tVeloIter(adjustedVelocities);
	for(forcesIter.Init(), veloIter.Init(), tVeloIter.Init(); !forcesIter.IsDone(); 
		forcesIter.Next(), veloIter.Next(), tVeloIter.Next())
	{
		vDotF = 0;
		fSq = 0;
		veloCurr = veloIter.Current();
		tVeloCurr = tVeloIter.Current();
		forcesCurr = forcesIter.Current();
		for (coord=0; coord<DIMENSION; coord++)
		{
			vDotF += veloCurr[coord]* forcesCurr[coord];
			fSq += forcesCurr[coord] * forcesCurr[coord];
		}
		if(vDotF < 0)
			for(coord=0;coord<DIMENSION;coord++) tVeloCurr[coord] = 0.; 
		else if(fSq > 1.0e-10)
		{
			fSqInv = 1.0/fSq;
			for(coord=0;coord<DIMENSION;coord++)
			    tVeloCurr[coord] = forcesCurr[coord] * vDotF * fSqInv;		
		}
	}
	
}
