// VerletThermalizeAtomsMover.h: interface for the VerletThermalizeAtomsMover class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

// Randomize kinetic energies to current temperature, then run nSteps of the
// Verlet algorithm

#ifndef VERLETTHERMALIZEATOMSMOVER_H
#define VERLETTHERMALIZEATOMSMOVER_H

#include "AtomsMover.h"
#include "Potential.h"
#include "Rand.h"

class VerletThermalizeAtomsMover : public AtomsMover  
{
public:
	VerletThermalizeAtomsMover(Potential *potential = 0, double temperature = 0.0, double dt = 0.01, int nSteps = 1);
	virtual ~VerletThermalizeAtomsMover() {}
	void Move(ListOfAtoms *atoms);

	// Time step for Verlet algorithm
	double GetDt() const {return dt;}
	void SetDt(double dt) {this->dt = dt;}

	// Target temperature
	double GetTemperature() const {return temperature;}
	void SetTemperature(double temperature) {this->temperature = temperature;}

	// Number of time-steps per call.
	int GetNSteps() const {return nSteps;}
	void SetNSteps(int nSteps) {this->nSteps = nSteps;}
	
	// Re-initialize random number generator
	void SetRandomSeed(int seed) 
	{
		rand.SetSeed(seed);
	}

	int GetRandomSeed() const
	{
		return rand.GetOriginalSeed();
	}

#ifndef SWIG
    void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
    {
        return className;
    }
#endif // SWIG

protected:
	void Allocate(int nAtoms, const DMArray<double, DIMENSION> *array);
	double dt;
	int nAtoms;		// Tells how large a force array is stored
	int nSteps;
	DMArray<double, DIMENSION> *forces;
	DMArray<double, DIMENSION> *newVelocities;

	double temperature; // Target temperature
	Rand rand;			// Random number generator
	static const std::string className;
};

#endif // !defined(VERLETTHERMALIZEATOMSMOVER_H)
