// DTUPeriodicBoundaryConditions.h: interface for the DTUPeriodicBoundaryConditions class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef DTUPERIODICBOUNDARYCONDITIONS_H
#define DTUPERIODICBOUNDARYCONDITIONS_H

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // SWIG

#include "PeriodicBoundaryConditions.h"

// XXX Defines are ugly. This one doesn't seem very necessary.
#define ASSUREPOSI 5.0


//! This does the same job as SimplePeriodicBoundaryConditions, but uses rounding on scaled coordinates. Machine dependent!

/**
 * The enforcing of periodic boundary conditions is carried out by first 
 * scaling the coordinates (using BLAS) so that they lie between 0 and 1.
 * Then the integer part is subtracted (there's an extra half in the case of
 * DifferenceBoundaryConditions). One can either use  (int) (expression) to get
 * the integer part of expression, or on some architectures (IEEE compliant, I
 * believe) add and subtract a certain "magic" number--thus avoiding switching
 * between floating point and integer units. I seem to have been experimenting
 * with both. I must figure out which is the most appropriate.
 *
 * For now it is assumed that coordinates run from 0 to length[coord] in each
 * direction. In the future, it will allow a min and a max = min + length in 
 * each direction (which will change EnforceBoundaryConditions but not
 * DifferenceBoundaryConditions)
 *
 * \ingroup moleculardynamics
 * \author nbailey
 * \date 5/16/2000
 */



class DTUPeriodicBoundaryConditions : public PeriodicBoundaryConditions  
{
public:
	DTUPeriodicBoundaryConditions();
	virtual ~DTUPeriodicBoundaryConditions();
	void EnforceBoundaryConditions(DMArray *positions) const;
	void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const;
	void EnforceBoundaryConditions(double *position) const;
	void SetLength(const double l[DIMENSION])
	{
		// somewhere it should be checked that the length of the box is not too 
		// small compared with the cutoffs used in the simulation (maybe at outer level)
		for (int coord = 0; coord < DIMENSION; coord++)
		{
			length[coord] = l[coord];
			invLength[coord] = 1.0/l[coord];
		}
	}
	BoundaryConditions *Copy() const;
private:
	double invLength[DIMENSION];
	double assureposi;
	double assureposihalf;
};

#endif // !defined(DTUPERIODICBOUNDARYCONDITIONS_H)
