// 3DLattices.h: interfaces for the 3DLattices
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DMFirstHeader.h"

#include "DigitalMaterial.h"

#include "BravaisLatticeWithBasis.h"
#if DIMENSION == 3


class CubicLattice: public BravaisLatticeWithBasis
{
public:
    void SetLatticeConstant(double a);
    double GetLatticeConstant() const {return latticeConstant;}
    CubicLattice(double a);
    ~CubicLattice(){};
    const std::string &GetType() const {return className;}
protected:
    double latticeConstant;
private:
	static const std::string className;
};


class FCCLattice:public CubicLattice
{
public:
    FCCLattice(double a=1.0);
    ~FCCLattice(){}
    void AddAtomInCell(TinyVector<double, DIMENSION> atomCoord,
                               std::string name="");
    const std::string &GetType() const {return className;}
private:
	static const std::string className;
};

class PrimitiveFCCLattice:public BravaisLatticeWithBasis
{
public:
    PrimitiveFCCLattice(double a=1.0);
    ~PrimitiveFCCLattice(){};
    //! restores the usual definition of the lattice vectors.
    void ResetOrientation(); 
    const std::string &GetType() const
	{
		return className;
	}
    void SetLatticeConstant(double a);
    double GetLatticeConstant() const {return latticeConstant;}
protected:
    //! The lattice constant has the same meaning as in FCCLattice
    double latticeConstant;
private:
	static const std::string className;
};

class BCCLattice:public CubicLattice
{
public:
    BCCLattice(double a=1.0);
    void AddAtomInCell(TinyVector<double, DIMENSION> atomCoord,
                       std::string name="");
    const std::string &GetType() const
	{
		return className;
	}
private:
	static const std::string className;
};

class DiamondLattice: public FCCLattice
{
public:
    //! Note that the lattice spacing is the one of the uderlying cubic lattice
    DiamondLattice(double a=1.0);
	~DiamondLattice(){}
    void AddAtomInCell(TinyVector<double, DIMENSION> atomCoord,
                       std::string name="");
    const std::string &GetType() const {return className;}
private:
	static const std::string className;
};

class TriclinicLattice:public GeneralLattice
{
public:
    TriclinicLattice(const double latticeConstants[DIMENSION],
                     const double anglesInUnitCell[DIMENSION]);
    //! Default constructor needed for serialization
    TriclinicLattice();
    ~TriclinicLattice(){};

    void SetLatticeConstants(const double latticeConstants[DIMENSION]);
    void SetAnglesInUnitCell(const double anglesInUnitCell[DIMENSION]);

    const double (&GetLatticeConstants()const) [DIMENSION] {return latticeConstants;}
    const double (&GetAnglesInUnitCell()const) [DIMENSION] {return anglesInUnitCell;}

    //! Load the class.
    void Load(DMReader *reader) ;

    //! Get the string name of the class for saving or loading.
    const std::string &GetType() const {return className;}
protected:

    double latticeConstants[DIMENSION];
    double anglesInUnitCell[DIMENSION];
    void Set3DLatticeVectors();
private:
	static const std::string className;
};


#endif // DIMENSION == 3
