// ExternalForceConstraint.h: interface for the ExternalForceConstraint class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef EXTERNALFORCECONSTRAINT_H
#define EXTERNALFORCECONSTRAINT_H

#include "DigitalMaterial.h"
#include "Constraint.h"

//! Allows external forces to be added to a ListOfAtoms

/*!

  This constraint allows you to add an external force (when you call 
  AdjustForces). So it can be used on a ListOfAtoms forming a boundary, say,
  where external force or stress is to be applied.

  Degrees of freedom are otherwise unconstrained so no need to overload the
  GeneralizedCoordinate functions.

  Previously (before the date below) existed as the class "Handle" in 
  Constraint.h

  
  \ingroup MolecularDynamics
  \author Nicholas Bailey
  \date March 1, 2001
  
 */ 



class ExternalForceConstraint : public Constraint
{
public:
	ExternalForceConstraint(); 
	virtual ~ExternalForceConstraint() ;
	Constraint *Copy() const;
	void SetExternalForce(const double extForce[DIMENSION])
		{for(int coord=0;coord<DIMENSION;++coord) externalForce[coord]=extForce[coord];}
	const double *GetExternalForce() const {return externalForce;}
	void AdjustForces(DMArray<double, DIMENSION> *forces);


	const std::string &GetType() const
	{
		return className;
	}

protected:
	double externalForce[DIMENSION];
	static const std::string className;
};


#endif // !defined(EXTERNALFORCECONSTRAINT_H)
