// FixedBody.cpp: implementation of the FixedBody class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"

#include "FixedBody.h"
#include "DMArray.h"

#include "DMLastHeader.h"


const std::string FixedBody::className = std::string("FixedBody");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



FixedBody::FixedBody(const DMArray<double, DIMENSION> *positions)
{
	fixedPositions = 0;
	SetFixedPositions(positions);
	for(int i = 0; i < DIMENSION; i++)
		totalForce[i] = 0.0;
}

FixedBody::~FixedBody()
{
	if(fixedPositions) delete fixedPositions;
}

Constraint *FixedBody::Copy() const
{
	return new FixedBody(fixedPositions);
}

void FixedBody::SetFixedPositions(const DMArray<double, DIMENSION> *positions)
{
	if(fixedPositions) delete fixedPositions;
	fixedPositions = new DMArray<double, DIMENSION> (positions);
	fixedPositions->Copy(positions);
}

void FixedBody::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
	if(!(positions->GetShape() == fixedPositions->GetShape()))
		throw DMException("FixedBody","AdjustPositions","Shape of positions differs from that of fixedPositions");
	ASSERT(positions->GetShape() == fixedPositions->GetShape());
	positions->Copy(fixedPositions);
}

void FixedBody::AdjustForces(DMArray<double, DIMENSION> *forces)
{
	forces->Sum(totalForce);
	forces->Zero();
}

double FixedBody::GetTotalForce(int coord)
{
	return totalForce[coord];
}

int FixedBody::NumberGeneralizedCoordinates()
{
	return 0;
}

void FixedBody::SetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void FixedBody::GetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void FixedBody::IncrementGeneralizedCoordinate(int n,double inc)
{
	
}


void FixedBody::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{

}

void FixedBody::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{

}
