// FixedInLine.h: interface for the FixedInLine class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef FIXEDINLINE_H
#define FIXEDINLINE_H

#include "DigitalMaterial.h"
#include "Constraint.h"
#include "FixedBody.h"

class FixedInLine : public Constraint
{
public:
	FixedInLine(const DMArray<double, DIMENSION> *positions, const double tangent[DIMENSION]);
        ~FixedInLine();
	Constraint *Copy() const;

	void SetFixedPositions(const DMArray<double, DIMENSION> *positions);
	void SetTangent(const double tangent[DIMENSION]);
	void AdjustPositions(DMArray<double, DIMENSION> *positions);
	void AdjustForces(DMArray<double, DIMENSION> *forces);
	double GetTotalForce(int coord);

	int NumberGeneralizedCoordinates();
	// these all do nothing, since there are no generalized coordinates
	// we don't want them to throw errors because they will be involved
	// in loops over branches, in which case they will be passed over
	// and the arrays wfilled in/used by the next leaf/branch which has
	// actual degrees of freedom
	void SetGeneralizedCoordinates(double *generalizedCoordinates) ;
    void GetGeneralizedCoordinates(double *generalizedCoordinates); 
	void IncrementGeneralizedCoordinate(int n,double inc);
#ifndef SWIG
    void CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces);
    void DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION]);
#endif // SWIG
	const std::string &GetType() const
	{
		return className; 
	}


protected:
	DMArray<double, DIMENSION> *fixedPositions;
	double fixedLineTangent[DIMENSION];
	static const std::string className;
	double totalForce[DIMENSION];
};



#endif // !defined(PARTIALFIXEDBODY_H)
