// TimeDependentConstraint.cpp: implementation of the TimeDependentConstraint class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"

#include "TimeDependentConstraint.h"
#include "DMArray.h"

#include "DMLastHeader.h"

const std::string TimeDependentConstraint::className = std::string("TimeDependentConstraint");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


TimeDependentConstraint::TimeDependentConstraint(double initialTime, double timeStep, const DMArray<double, DIMENSION> * positions, Functor *func)
{
  positionsZero = NULL;
  Reset(initialTime,timeStep,positions);
  this->func = func;
}


TimeDependentConstraint::~TimeDependentConstraint()
{
  if(positionsZero) delete positionsZero;
}


void TimeDependentConstraint::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
  double functionValue[DIMENSION];
  func->Evaluate(time,functionValue);
  positions->Copy(positionsZero);

  DMArrayIterator<double, DIMENSION> it(positions);
  for(it.Init();!it.IsDone(); it.Next())
    {
      for(int coord=0;coord<DIMENSION;++coord)
		it.Current()[coord] += functionValue[coord];
  }	 
  time += timeStep;
}


void TimeDependentConstraint::Reset(double initialTime, double timeStep, const DMArray<double, DIMENSION> *positions)
{
  time = initialTime;
  this->timeStep = timeStep;

  if(positionsZero) delete positionsZero;
  positionsZero = new DMArray<double, DIMENSION> (positions);
  positionsZero->Copy(positions);
}




Constraint *TimeDependentConstraint::Copy() const
{
  return new TimeDependentConstraint(time, timeStep, positionsZero, func);
}


int TimeDependentConstraint::NumberGeneralizedCoordinates()
{
	return 0;
}

void TimeDependentConstraint::SetGeneralizedCoordinates(double *generalizedCoordinates)
{

}

void TimeDependentConstraint:: GetGeneralizedCoordinates(double *generalizedCoordinates)
{

}

void TimeDependentConstraint::IncrementGeneralizedCoordinate(int n,double inc)
{

}

void TimeDependentConstraint::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{

}

void TimeDependentConstraint::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{

}
