// ArrayMathOps.h inlined math ops for DMArrays
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

template<>
inline void DMArray<double, DIMENSION>::Sum(double sum[DIMENSION])const
{
	memset(sum, 0, DIMENSION*sizeof(double));
	int size = cache.sizes.size();
	for(int i=0; i<size; ++i) 
	{
		int length = cache.sizes[i];
		double (*ptr)[DIMENSION] = cache.ptrs[i];
		for(int j=0; j<length; ++j)
			for(int coord=0; coord<DIMENSION; coord++)
				sum[coord] += ptr[j][coord];
	}
#ifdef DMPARALLEL
	double partialSum[DIMENSION];
	memcpy(partialSum, sum, DIMENSION*sizeof(double));
    dmProtocol->AllReduce(partialSum, sum, DIMENSION, DMProtocol::SUM);
#endif
}

template<>
inline void DMArray<double, DIMENSION>::Average(double avg[DIMENSION]) const
{
	double sum[DIMENSION+1];
	memset(sum, 0, DIMENSION*sizeof(double));
	int size = cache.sizes.size();
	for(int i=0; i<size; ++i) 
	{
		int length = cache.sizes[i];
		double (*ptr)[DIMENSION] = cache.ptrs[i];
		for(int j=0; j<length; ++j)
			for(int coord=0; coord<DIMENSION; coord++)
				sum[coord] += ptr[j][coord];
	}
	sum[DIMENSION] = (double) Size();
#ifdef DMPARALLEL
	double partialSum[DIMENSION+1];
	memcpy(partialSum, sum, (1+DIMENSION)*sizeof(double));
    dmProtocol->AllReduce(partialSum, sum, DIMENSION+1, DMProtocol::SUM);
#endif
	for(int coord=0; coord<DIMENSION; ++coord)
		avg[coord] = sum[coord] / sum[DIMENSION];
}

