// CheckPointObserver.cpp: implementation of the CheckPointObserver class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


#include "DigitalMaterial.h"
#include "CheckPointObserver.h"
#include "ListOfAtoms.h"
#include "NetCDFWriter.h"
#include "SprintString.h"
#include "BoundaryConditions.h"
#include "NeighborLocator.h"
#include "Potential.h"
#include "AtomsMover.h"

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CheckPointObserver::CheckPointObserver(Potential *potential, AtomsMover *mover, std::string baseFileName, bool oneFile, int notifyLevel) : ListOfAtomsObserver(notifyLevel),lastFileName("")
{
	this->potential = potential;
	this->mover = mover;
	this->baseFileName = baseFileName;
	this->oneFile = oneFile;
	this->counter = 0;
	
	if(oneFile)
		writer = new NetCDFWriter(baseFileName);
	else
		writer = 0;
}

CheckPointObserver::~CheckPointObserver()
{
	if(oneFile)
		delete writer;
}


void CheckPointObserver::Update(ListOfAtoms *atoms)
{

	SprintString suffix(".%.5d",counter);
	// frameName only used oneFile is not true, but need
	// to declare it outside the if statement
	string frameName(baseFileName);
	frameName += suffix;

	
	string LOAName("LOA");
	string BCName("BC");
	string NLName("NL");
	string POTName("POT");
	string MOVName("MOV");

	// if using one file, the frame numbers have to be incorporated into the object names
	if( oneFile )
	{
		LOAName += suffix;
		BCName += suffix;
		NLName += suffix;
		POTName += suffix;
		MOVName += suffix;
	}
	else
	{
		writer = new NetCDFWriter(frameName);
		// cout << frameName << endl;
	}
	


	writer->Put(atoms,LOAName);
	writer->Put(atoms->GetBoundaryConditions(),BCName);
	writer->Put(atoms->GetNeighborLocator(),NLName);
	writer->Put(potential, POTName);
	writer->Put(mover, MOVName);
	
	if( !oneFile )
	{
		delete writer;	
		// it seems wise to only assign lastFileName after successfully
		// doing the save
		lastFileName = frameName;
	}
	++counter;
}

