// CheckPointRestarter.cpp: implementation of the CheckPointRestarter class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


#include "DigitalMaterial.h"
#include "CheckPointRestarter.h"
#include "ListOfAtoms.h"
#include "NetCDFReader.h"
#include "SprintString.h"
#include "BoundaryConditions.h"
#include "NeighborLocator.h"
#include "Potential.h"
#include "AtomsMover.h"

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CheckPointRestarter::CheckPointRestarter(std::string fileName)
{
	reader = new NetCDFReader(fileName);

	potential = 0;
	mover = 0;
	boundaryConditions = 0;
	neighborLocator = 0;
}

CheckPointRestarter::~CheckPointRestarter()
{

}

void CheckPointRestarter::Recreate()
{

	// read the objects off the file
	atoms = dynamic_cast<ListOfAtoms*>(reader->Get("LOA"));
	boundaryConditions= dynamic_cast<BoundaryConditions*>(reader->Get("BC"));
	neighborLocator = dynamic_cast<NeighborLocator*>(reader->Get("NL"));
	potential = dynamic_cast<Potential*>(reader->Get("POT"));
	mover = dynamic_cast<AtomsMover *>(reader->Get("MOV"));
	
	// now reconnect everything
	atoms->SetBoundaryConditions(boundaryConditions);
	atoms->SetNeighborLocator(neighborLocator);
	mover->SetPotential(potential);

	// This doesn't work for all movers! In particular it won't work for a mover which has a pointer to another object. CompositeAtomsMover is one such, but it has not been developed very far anyway. CompositeLangevinAtomsMover will work when I have added in code to save the friction array.

}
