// CheckPointRestarter.h: interface for the CheckPointRestarter class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef CHECKPOINTRESTARTER_H
#define CHECKPOINTRESTARTER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "DMReader.h"
#include <string>


class DMWriter;
class Potential;
class AtomsMover;
class ListOfAtoms;
class NeighborLocator;
class BoundaryConditions;

//! A class which can restart a simulation given the filename corresponding to a particular checkpoint



/*!

  
  \author nbailey
  \date 8/1/2001
  \ingroup moleculardynamics

 */

class CheckPointRestarter 
{
public:
	CheckPointRestarter(std::string fileName);
	void Update(ListOfAtoms *atoms);
	virtual ~CheckPointRestarter();

	//! Recreate reads creates the object from the information in the file and attaches them to each other (sets the various pointers)
	void Recreate();
	
	//! Having called Recreate(), the actual objects can be obtained by the following functions
	Potential *GetPotential() { return potential; }
	AtomsMover *GetMover() { return mover; }
	ListOfAtoms *GetListOfAtoms () { return atoms; }
	BoundaryConditions *GetBoundaryConditions() { return boundaryConditions; }
	NeighborLocator *GetNeighborLocator() { return neighborLocator; }

private:
	DMReader *reader;
	ListOfAtoms *atoms;
	BoundaryConditions *boundaryConditions;
	NeighborLocator *neighborLocator;
	Potential *potential;
	AtomsMover *mover;
};

#endif // !defined(CHECKPOINTRESTARTER_H)
