// CrackPlaneObserver.h: interface for the CrackPlaneObserver class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef CRACKPLANEOBSERVER_H
#define CRACKPLANEOBSERVER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"

#include "ListOfAtomsObserver.h"
#include <vector>

class CrackPlaneObserver : public ListOfAtomsObserver
{
public:
	CrackPlaneObserver(double planePoint[DIMENSION], double planeNormal[DIMENSION], double cutoffDistance, int notifyLevel=0);
	virtual ~CrackPlaneObserver();
	void MakePairList(ListOfAtoms *atoms);
	void Update(ListOfAtoms *atoms);



	double GetMaxIncrease() { return maxIncrease; }
	double GetMeanIncrease() { return meanIncrease; }
	// returns perpendicular distance from plane with a minus if on the opposite side to the positive of the normal
	double GetDistanceFromPlane(double position[DIMENSION]) const;
	

	const double *GetPlanePoint() const { return planePoint; }
	void SetPlanePoint(double planePoint[DIMENSION]);
	const double *GetPlaneNormal() const { return planeNormal; }
	void SetPlaneNormal(double planeNormal[DIMENSION]); 

	const double GetCutoffDistance() const { return cutoffDistance; }
	void SetCutoffDistance(double cutoffDistance) { this->cutoffDistance = cutoffDistance; } 

#ifndef SWIG
	const std::vector<double> &GetPairSeparations() const { return pairSeparations; }
	const std::vector<std::pair<int, int> > &GetPairList() const { return pairList; } 

private:
	double planePoint[DIMENSION];
	double planeNormal[DIMENSION];
	double cutoffDistance;

	double maxIncrease;
	double meanIncrease;
	
	std::vector<std::pair<int, int> > pairList;
	std::vector<double> pairSeparations;
	std::vector<double> initialSeparations;
#endif // SWIG
};

#endif // !defined(CRACKPLANEOBSERVER_H)
