// LocalForceObserver.h: interface for the LocalForceObserver class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef LOCALFORCEOBSERVER_H
#define LOCALFORCEOBSERVER_H

#include "DMFirstHeader.h"
#include <string>
#include <fstream>
#include "DigitalMaterial.h"
#include "ListOfAtomsObserver.h"
#include "DMArray.h"

class Potential;
template<class Type, int N> class DMArray ;

class LocalForceObserver : public ListOfAtomsObserver
{
public:
	LocalForceObserver(Potential *potential, double forceCutoff, std::string fileName, int notifyLevel = 0);
	virtual ~LocalForceObserver();
	void Update(ListOfAtoms *atoms);

private:
	void Allocate(ListOfAtoms *atoms);
	int nAtoms;
	Potential * potential;
	DMArray<double, DIMENSION> *forces;
	std::ofstream *fileObject;
	double forceCutoff;
};

#endif // !defined(LOCALFORCEOBSERVER_H)
