// StressIntensityObserver.h: interface for the StressIntensityObserver class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef STRESSINTENSITYOBSERVER_H
#define STRESSINTENSITYOBSERVER_H

#include "DigitalMaterial.h"
#include "ListOfAtomsObserver.h"
#include "DMArray.h"

class Potential;
class NeighborLocator;
class BravaisLatticeWithBasis;
class NotchFieldDisplacer;

class StressIntensityObserver : public ListOfAtomsObserver
{
public:
	void SetPotential(Potential *potential);
	void SetNeighborLocator(NeighborLocator *neighborLocator);
	void SetLattice(BravaisLatticeWithBasis *lattice);
	void SetPathCenter(double pathCenter[DIMENSION]);
	void SetPathRadius(double pathRadius);

	void Update(ListOfAtoms *atoms);
	StressIntensityObserver(Potential *potential, NeighborLocator *neighborLocator, BravaisLatticeWithBasis *lattice, NotchFieldDisplacer *comlimentaryField, double pathCenter[DIMENSION], double pathRadius, double heightTolerance, const DMArray<double, DIMENSION> *undeformedPositions, int notifyLevel = 0);
	virtual ~StressIntensityObserver();

	double GetH_Integral() {return H_integral;}

private:
	Potential * potential;
	NeighborLocator *neighborLocator;
	BravaisLatticeWithBasis *lattice;
	NotchFieldDisplacer *complimentaryField;
	double pathCenter[DIMENSION];
	double pathRadius;
	double heightTolerance;
	double H_integral;

	int stresscomp[DIMENSION][DIMENSION];
	const DMArray<double, DIMENSION> *undeformedPositions;
};

#endif // !defined(STRESSINTENSITYOBSERVER_H)
