// DomainSubdivision.h: interface for the DomainSubdivision class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


// Two related tasks:
//   1: Decide on which processor an atom should reside.
//   2: Decide which processors need to know about an atom that resides on
//      this processor.
#ifndef DOMAINSUBDIVISION_H
#define DOMAINSUBDIVISION_H

#include "DMFirstHeader.h"
#include <vector>

#include "DigitalMaterial.h"

template<class Type, int N> class DMArray;
class Index;

class DomainSubdivision 
{
public:
	DomainSubdivision(const int cpuLayout[DIMENSION],
							  int myDomain);
	virtual ~DomainSubdivision() {};
	//! Ask the domain subdivision to adjust its parameters according to these positions
	virtual void Update(const DMArray<double, DIMENSION> *positions) {};
	virtual int WhichDomain(const double point[]) const = 0; 
	virtual bool PointNearDomain(const double point[DIMENSION], int domain) const = 0;
	virtual void FindPointsNearDomain(const DMArray<double, DIMENSION> *positions,
									  std::vector<Index> *interestingAtomsToSend) const = 0;
protected:
	int numberOfProcs;
	int myDomain;
	int cpuLayout[DIMENSION];
};


#endif // !ndef ABSRACTDOMAINSUBDIVISION_H
