// ParallelAtomsInitializer.cpp: implementation of the ParallelAtomsInitializer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"


#include "ListOfAtoms.h"
#include "ParallelAtomsInitializer.h"
#include "DMProtocol.h"
#include "ProtocolFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParallelAtomsInitializer::ParallelAtomsInitializer(AtomsInitializer *initializer)
{
    this->initializer = initializer;
}

void ParallelAtomsInitializer::Create(ListOfAtoms *atoms)
{
    const int myProc = dmProtocol->GetProcessorNumber();
    const int nProcs = dmProtocol->GetNumberOfProcessors();
    DMProtocol::Buffers *buffers = dmProtocol->NewBuffers();
    if(myProc == 0)
    {
        ProtocolFactory::Instance()->SetProtocol(ProtocolFactory::Local);
        initializer->Create(atoms);
        ListOfAtoms *tmp = atoms->Copy(); // we may invent another mechanism.
        for(int leaf = 0; leaf < tmp->NumberOfLeaves(); ++leaf)
            tmp->GetLeaf(leaf)->SetShape(0); // empty the leaves before sending them
        for(int proc=1; proc < nProcs; ++proc)
            tmp->Save(buffers->GetSendBuffer(proc));
        delete tmp;
        ProtocolFactory::Instance()->Reset();
    }

    buffers->AllToAll();
    
    if(myProc != 0)
    {
        atoms->Load(buffers->GetRecvBuffer(0));
    }
    delete buffers;
}
