// ParallelPotential.cpp: implementation of the ParallelPotential class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"
#include "DMArray.h"
#include "ParallelPotential.h"
#include "DMProtocol.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParallelPotential::ParallelPotential(Potential *potential)
{
    this->potential = potential;
}

double ParallelPotential::CalculateEnergy(ListOfAtoms *atoms)
{
    double totalEnergy;
    double myEnergy = potential->CalculateEnergy(atoms);
    dmProtocol->AllReduce(&myEnergy, &totalEnergy, 1, DMProtocol::SUM);
    return totalEnergy;
}

void ParallelPotential::CalculateForces (ListOfAtoms *atoms,
                                         DMArray<double, DIMENSION> *forces)
{
    forces->GetSynchronizationKit()->Init();
    potential->CalculateForces(atoms, forces->GetSynchronizationKit()->GetExtended());
}

void ParallelPotential::CalculateForcesBetweenTypes(ListOfAtoms *atoms,
                          DMArray<double, DIMENSION> *forces, int type1, int type2)
{
    forces->GetSynchronizationKit()->Init();
    potential->CalculateForcesBetweenTypes(atoms, 
        forces->GetSynchronizationKit()->GetExtended(), type1, type2);
}

double ParallelPotential::CalculateEnergyAndForces(ListOfAtoms *atoms, 
                                                   DMArray<double, DIMENSION> *forces)
{
    forces->GetSynchronizationKit()->Init();
    double totalEnergy;
    double myEnergy = potential->CalculateEnergyAndForces(atoms, 
        forces->GetSynchronizationKit()->GetExtended());
    dmProtocol->AllReduce(&myEnergy, &totalEnergy, 1, DMProtocol::SUM);
    return totalEnergy;
}

double ParallelPotential::CalculateAtomicEnergy(ListOfAtoms *atoms, int i)
{
    return potential->CalculateAtomicEnergy(atoms, i);
}