// CompositePotential.h: interface for the CompositePotential class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef COMPOSITEPOTENTIAL_H
#define COMPOSITEPOTENTIAL_H

#include "DMFirstHeader.h"

#include "DigitalMaterial.h"
#include "Potential.h"

class CompositePotential : public Potential
{
public:
	CompositePotential() {potential1=0; potential2=0;};
	CompositePotential(Potential *potential1, Potential *potential2);
	~CompositePotential();

	void CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces);
	double CalculateEnergy(ListOfAtoms *atoms);

	double GetLengthScale() const; // this is arbitrary !!!
	double GetEnergyCurvatureScale() const ;// Arbitrary! Should be zero.
	double CalculateAtomicEnergy(ListOfAtoms *atoms, int i);
//	double GetInnerCutoffDistance() const;
	
	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);

	void Allocate(const DMArray<double, DIMENSION> *array);

	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}

#ifndef SWIG
private:
	Potential *potential1;
	Potential *potential2;
 	DMArray<double, DIMENSION> *forces2;

	static const std::string className;
#endif // SWIG 
};

#endif // !defined(COMPOSITEPOTENTIAL_H
