// EMTDefaultParameterProvider.h  --  provides the default EMT parameters.

/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _EMTDEFAULTPARAMETERPROVIDER_H
#define _EMTDEFAULTPARAMETERPROVIDER_H

#include "DMFirstHeader.h"
#include "EMTParameterProvider.h"
#include <vector>

class EMTDefaultParameterProvider : public EMTParameterProvider
{
public:
	EMTDefaultParameterProvider();
	virtual ~EMTDefaultParameterProvider();
	virtual const emt_parameters *GetParameters(const std::string *element);
	virtual void CalcGammaEtc();
    // Can the cutoff be made element-dependent?
	virtual double GetCutoffDistance() {return cutoff;}
	virtual double GetCutoffSlope() {return cutslope;}
	// The potential delegates GetLengthScale to the provider.
	// What should be done for multiple elements?
	virtual double GetLengthScale() {return params[0]->lengthscale;}
	virtual int GetNumberOfElements() {return params.size();}
	virtual const TinyDoubleMatrix *GetChi() {return chi;}
	virtual void Debug();
	
#ifndef SWIG   // SWIG cannot parse all of this
protected:
	virtual emt_parameters *GetNewParameters(const std::string element);
	virtual void calc_cutoff();
	virtual void calc_gammas();
	virtual void calc_chi();
	
	std::vector<emt_parameters *> params;
	TinyDoubleMatrix *chi;
	double maxseq;
	double cutoff;
	double listcutoff;   // Will this be used ??
	double cutslope;

	// Two constants: the last shell included in interaction range, and in nb list
	static const int shell0;
	static const int shell1;
#endif
};

#endif // ! _EMTDEFAULTPARAMETERPROVIDER_H
