// GravityPotential.h: interface for the GravityPotential class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef GRAVITYPOTENTIAL_H
#define GRAVITYPOTENTIAL_H

#include "DMFirstHeader.h"

#include "DigitalMaterial.h"
#include "Potential.h"

class GravityPotential : public Potential
{
public:
	GravityPotential();
	virtual ~GravityPotential() {};

	void CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces);
	double CalculateEnergy(ListOfAtoms *atoms);	

	double GetG() const {return g;}
	void SetG(double g);

 	void SetDirection(double direction[DIMENSION]);


	double GetLengthScale() const {return 1.;} // this is arbitrary !!!
	double GetEnergyCurvatureScale() const {return 1.;} // Arbitrary! Should be zero.
	double CalculateAtomicEnergy(ListOfAtoms *atoms, int i) {return 0;}
	
	void Save(DMWriter *writer) const
		{    
			double doubleDir[DIMENSION];
			std::vector<int> dim;
			dim.push_back(DIMENSION);
			writer->PutDoubleArray(doubleDir, dim, "direction");
			writer->PutDouble(g,"g");
		}
	void Load(DMReader *reader)
		{
			std::vector<int> dim;
			double *data = reader->GetDoubleArray(dim, "direction");
			if((dim.size() != 1) || (dim[0] != DIMENSION)) 
				throw DMException("GravityPotential", "Load", "Problem while reading direction");
			for(int coord=0; coord<DIMENSION; ++coord) direction[coord] = data[coord];
			delete data;
			g = reader->GetDouble("g");
		}

	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}

#ifndef SWIG
private:
	double g;
	double direction[DIMENSION];

	static const std::string className;
#endif // SWIG 
};

#endif // !defined(GRAVITYPOTENTIAL_H
