// AnisotropicMultipoleField.h: interface for the AnisotropicMultipoleField class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef ANISOTROPICMULTIPOLEFIELD_H
#define ANISOTROPICMULTIPOLEFIELD_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <complex>

#include "MultipoleField.h"


class AnisotropicMultipoleField : public MultipoleField
{
public:
	AnisotropicMultipoleField(int maximumOrder, const double direction[DIMENSION], double center[DIMENSION],double *C);
	virtual ~AnisotropicMultipoleField();

	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);
	void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);

#ifndef SWIG
	void GradientWRTParameters(const double position[DIMENSION], double (*displacementGradient)[DIMENSION]);

private:
	double elasticConstants[3]; // assuming cubic symmetry: 3 elastic constants (C11, C12, C44)	
	std::complex<double> anisotropyPhaseFactor; // alpha in Eshelby, Sinclair papers
	std::complex<double> yCompFactor; // A in Eshelby, Sinclair papers
#endif //SWIG
};

#endif // !defined(ANISOTROPICMULTIPOLEFIELD_H)
