// CopySubset.cpp: implementation of the CopySubset class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"
#include "CopySubset.h"
#include "AtomsLocator.h"
#include <vector>
#include <algorithm>

#include "DMLastHeader.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CopySubset::CopySubset( AtomsLocator *locator)
{
	this->locator = locator;
}

CopySubset::~CopySubset()
{

}


/*! The Transform function makes a copy of the positions, iterates over the 
  atom numbers in listOfAtomNumbers, gets the position of that atom in the
  copied array, calculates the displacement, and increments
  the position (in the copy). Then we use SetCartesianPositions to change
  the positions in the LOA. I'm assuming this is faster than using
  IncrementCartesianPosition for each atom to be moved--the neighborLocator is 
  called just once, but all atoms outside the subset get copied over 
  identically

  
  It would be great if we could set up a mechanism for
  IncrementCartesianPositions to work on a subset of the LOA, perhaps by 
  creating an LOA which is essentially a reference into the given one.


 */

ListOfAtoms *CopySubset::Copy(ListOfAtoms *atoms)
{
	std::vector<int> atomsToKeep = locator->GetAtomNumbers(atoms);
	sort(atomsToKeep.begin(),atomsToKeep.end());
	
	int nAtomsTotal = atoms->GetNumber();
	int nAtomsInSubset = atomsToKeep.size();


	if(nAtomsInSubset == 0)
		return new ListOfAtoms(0);
	

	ListOfAtoms *copy = atoms->Copy();
	
	std::set<int> atomsToRemove;
	int i = 0;
	std::vector<int>::const_iterator toKeepIter = atomsToKeep.begin();
	
	
	int nextAtomToKeep;
	
	while(toKeepIter != atomsToKeep.end())
	{

	    nextAtomToKeep = *toKeepIter;
		for ( int at = i; at < nextAtomToKeep; ++at )
			atomsToRemove.insert(at);


		i = nextAtomToKeep + 1;
		toKeepIter++;
	};
	// now include all the atoms after the last "to keep" one.
	for ( int at = nextAtomToKeep + 1; at < nAtomsTotal; ++at )
		atomsToRemove.insert(at);

	copy->RemoveAtoms(atomsToRemove);

	return copy;
}

