// CopySubset.h: interface for the CopySubset class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef COPYSUBSET_H
#define COPYSUBSET_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "ElasticFieldTransformer.h"
class AtomsLocator;


//! This will create a new ListOfAtoms which is a copy of a subset of the given ListOf Atoms


/*!

  You pass to the constructor a pointer to an AtomsLocator, which has a
  function GetAtomNumbers which provides a vector of the atom numbers to which
  the transformation is to be applied.

  \author nbailey
  \date 3/25/2002
  \ingroup moleculardynamics

 */


class CopySubset
{
public:
	CopySubset(AtomsLocator *locator);
	virtual ~CopySubset();

	ListOfAtoms * Copy(ListOfAtoms *atoms);
	
private:
#ifndef SWIG
	AtomsLocator *locator;
#endif // SWIG
};

#endif // !defined(COPYSUBSET_H)
