// DislocationMaker.h: interface for the DislocationMaker class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef DISLOCATIONMAKER_H
#define DISLOCATIONMAKER_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <assert.h>


#include "Quasi2DElasticField.h"
#include "ListOfAtoms.h"



class DislocationMaker : public Quasi2DElasticField
{
public:
	DislocationMaker(const double burgers[DIMENSION],const double direction[DIMENSION], double center[DIMENSION],const double nu,double thetaLowerCutoff=-PI);
	virtual ~DislocationMaker();




	// Gets and Sets for array member variables
	void SetBurgers(const double burgers[DIMENSION]); 
	const double *GetBurgers() const {return (const double *)burgers;}

	void SetDirection(const double direction[DIMENSION]); 
	const double *GetDirection() const {return (const double *)direction;}
	void SetCenter(const double center[DIMENSION]); 
	const double *GetCenter() const {return (const double *)center;}
	// Gets and Sets for nu
	void SetNu(double nu) {this->nu = nu;}
	double GetNu() const {return nu;}

#ifndef SWIG
	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);
	void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);


protected:
	double burgers[DIMENSION];
	double nu;
	double edge;
	double edge_over_2PI;
	double nu_factor;
	double screw;
	double edgeComponent[DIMENSION];
	virtual void UpdateMemberVariables();
#endif //SWIG
};

#endif // !defined(DISLOCATIONMAKER_H)
