// MultipoleField.h: interface for the MultipoleField class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


#ifndef MULTIPOLEFIELD_H
#define MULTIPOLEFIELD_H


#include "Quasi2DElasticField.h"

class MultipoleField : public Quasi2DElasticField
{
public:
	MultipoleField(int maximumOrder, const double direction[DIMENSION], double center[DIMENSION]):Quasi2DElasticField(direction,center)
		{
			this->maximumOrder = maximumOrder;
			coefficients = new double[maximumOrder][4];
			for(int n=0;n<maximumOrder;++n)
				for(int j=0;j<4;++j)
					coefficients[n][j] = 0.;
		}
	virtual ~MultipoleField()
		{
			delete [] coefficients;
		}
	virtual void GradientWRTParameters(const double position[DIMENSION], double (*displacementGradient)[DIMENSION])=0;

	void SetCoefficients(double (*coefficients)[4])
		{
			for(int n=0;n<maximumOrder;++n)
				for(int j=0;j<4;++j)
					this->coefficients[n][j] = coefficients[n][j];
		}
	
	void GetCoefficients(double (*coefficients)[4])
		{
			for(int n=0;n<maximumOrder;++n)
				for(int j=0;j<4;++j)
					coefficients[n][j] = this->coefficients[n][j];
		}
	int GetMaximumOrder() const {return maximumOrder;}

protected:
	int maximumOrder;
	double (*coefficients)[4];

};


#endif // !defined(MULTIPOLEFIELD_H)
