// SubsetElasticTransformer.h: interface for the SubsetElasticTransformer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SUBSETELASTICTRANSFORMER_H
#define SUBSETELASTICTRANSFORMER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "ElasticFieldTransformer.h"
class AtomsLocator;


//! This will apply an ElasticFieldTransformer to a subset (specified by atom numbers) of the LOA


/*!

  You pass to the constructor pointers to two objects: one is an
  ElasticFieldTransformer which specifies which kind of displacement is to be
  applied to the selected atoms. The other is an AtomsLocator, which has a
  function GetAtomNumbers which provides a vector of the atom numbers to which
  the transformation is to be applied.

  \author nbailey
  \date 7/10/2001
  \ingroup moleculardynamics

 */


class SubsetElasticTransformer : public ElasticFieldTransformer
{
public:
	SubsetElasticTransformer(ElasticFieldTransformer *transformerToApply, AtomsLocator *locator);
	virtual ~SubsetElasticTransformer();

	void Transform(ListOfAtoms *atoms);

	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);

    void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);
	
private:
#ifndef SWIG
	ElasticFieldTransformer *transformerToApply;
	AtomsLocator *locator;
#endif // SWIG
};

#endif // !defined(SUBSETELASTICTRANSFORMER_H)
