// NotchMaker.cpp: implementation of the WedgeMaker class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DigitalMaterial.h"
#include "WedgeMaker.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WedgeMaker::WedgeMaker(double direction[DIMENSION], double center[DIMENSION], double normal1[DIMENSION], double normal2[DIMENSION])
{

	for(int coord=0; coord<DIMENSION; coord++){
		this->direction[coord] = direction[coord];
		this->center[coord] = center[coord];
		this->normal1[coord] = normal1[coord];
		this->normal2[coord] = normal2[coord];
	}
}

WedgeMaker::~WedgeMaker()
{

}

void WedgeMaker::SetDirection(const double direction[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->direction[coord] = direction[coord];
}

void WedgeMaker::SetCenter(const double center[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->center[coord] = center[coord];
}

void WedgeMaker::SetNormal1(const double normal1[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->normal1[coord] = normal1[coord];
}

void WedgeMaker::SetNormal2(const double normal2[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->normal2[coord] = normal2[coord];
}

bool WedgeMaker::Inside(const double X[DIMENSION])
{
	bool inside;
	double perpDist[DIMENSION];
	double XdotDir = 0;
	double perpdotNorm1 = 0;
	double perpdotNorm2 = 0;

	for (int coord = 0; coord < DIMENSION; coord++)
		XdotDir += (X[coord]-center[coord])*direction[coord];
	for (int coord = 0; coord < DIMENSION; coord++){
		perpDist[coord] = X[coord] - center[coord] - XdotDir*direction[coord];
		perpdotNorm1 += perpDist[coord]*normal1[coord];
		perpdotNorm2 += perpDist[coord]*normal2[coord];
	}
	inside = ((perpdotNorm1 >= 0.0) && (perpdotNorm2 >= 0.0));

	return inside;
}

