// RasMolObserver.h: interface for the RasMolObserverCPP class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef RASMOLOBSERVER_H
#define RASMOLOBSERVER_H

#include <stdio.h> // for FILE
#include <map>
#include "ListOfAtomsObserver.h"
#include "ColorMethod.h"

//! Uses RasMol to plot the atoms
/*! It is  an observer of ListOfAtoms. Copied from PlotAtomsObserver

  Possibly would help if could get automatic temporary local files for the data, as is done in Python.

  \date Feb 1, 2001
  \author Nicholas Bailey

  */

class RasMolObserverCPP : public ListOfAtomsObserver  
{

public:

  //! Constructor.
	RasMolObserverCPP(int radius,ColorMethod *colorMethod=NULL);
  //! Destructor.
	virtual ~RasMolObserverCPP();

  //! Called by the subject to tell us when atoms move.
	void Update(ListOfAtoms *atoms);

  //! Alternative syntax for Update().
	void Plot(ListOfAtoms *atoms);

	void SetColor(int leaf,int color)
		{
			colorMap.insert(std::make_pair(leaf,color));
		}
	
	void SetColorScale ( double colorScale ) { this->colorScale = colorScale; colorScaleSetByUser = true; }
	void UnSetColorScale() { colorScaleSetByUser = false; }
	double GetColorScale() { return colorScale; }
	
	
	void SetLengthScale ( double lengthScale ) { this->lengthScale = lengthScale; }
	double GetLengthScale() { return lengthScale; }
protected:
#ifndef SWIG
	FILE*	        pipe_file; //!< Pipe to RasMol.
	int             radius;
	ColorMethod*    colorMethod;
	double colorScale;
	bool colorScaleSetByUser;
	std::map<int, int> colorMap; // maps leaf index to colour index

	double lengthScale;
#endif // SWIG	

};

#endif // !defined(RASMOLOBSERVER_H)
