// MDView.h : interface of the CMDView class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#if !defined(AFX_MDVIEW_H__4900D45F_41EA_45F3_AEF9_1050E548E567__INCLUDED_)
#define AFX_MDVIEW_H__4900D45F_41EA_45F3_AEF9_1050E548E567__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SubjectObserver.h"
#include "HWndStore.h"

class HWndStore;	// Avoids problems of recursion: CMDView refers to HWndStore which refers to CMDView.

#define WM_USER_UPDATE_SIM WM_USER+10

/*! \ingroup windows */
class CMDView : public CView , public Observer
{
protected: // create from serialization only
	CMDView();
	DECLARE_DYNCREATE(CMDView)

// Attributes
public:
	CMDDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMDView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL
	virtual void OnInitialUpdate();
	virtual void OnUpdate(CView *pSender, LPARAM lHint, CObject *pHint);
	virtual LRESULT OnUpdateSim(WPARAM wParam, LPARAM lParam);

// Implementation
public:	
	virtual ~CMDView();

	HWndStore * phWndStore;
	void Update(Subject *atoms);
	double const GetDiameter() {return diameter;}
	void SetDiameter(double diameter) {this->diameter = diameter;}
	double const GetPlottedRegionSize() {return plottedRegionSize;}
	void SetPlottedRegionSize(double maxSize);
	void SetCenter(const double center[DIMENSION]) 
	{
		for (int coord=0; coord<DIMENSION; coord++) this->center[coord]=center[coord];
	}

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CMDView)
	afx_msg void OnControlRestart();
	afx_msg void OnControlRestartPaused();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:
	ListOfAtoms* atoms;
	double plottedRegionSize;
	double center[DIMENSION];
	// From Matt Kuntz's program, Spring 1998
	double dX, dY;
	int window_width, window_height;
	CDC bufferDC;
	CDC *stored_pDC;	// For Update: don't get new device context each draw
	CBitmap *buffer;
	CBitmap *oldBuffer;
	double diameter;
};

#ifndef _DEBUG  // debug version in MDView.cpp
inline CMDDoc* CMDView::GetDocument()
   { return (CMDDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MDVIEW_H__4900D45F_41EA_45F3_AEF9_1050E548E567__INCLUDED_)
