// DialogBuilder.cpp : implementation file
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


#include "stdafx.h"
#include "MD.h"
#include "DialogBuilder.h"
#include "DialogIDManager.h"
#include "Thread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// DialogBuilder dialog

// XXX need to have a singleton idd_user base class to store these!
// Initialize static member variables for IDD
//const UINT DialogBuilder::idd_user_base = 0xa000;		// User-based IDD's start at 0x8000: potential problems with Win9X (see Prosise)
//UINT DialogBuilder::idd_current_user_number = 0xa000;

DialogBuilder::DialogBuilder(MDSimulation *pSim, Thread *simThread, CWnd* pParent /*=NULL*/)
{
	this->pSim = pSim;
	this->simThread = simThread;

//	idd_current_user_number ++;

	// I think we need to set IDD first before calling parent constructor?
	UINT IDD = DialogIDManager::Instance()->GetID();
	::CDialog(IDD, pParent);

	maxNumberOfFieldsPerArray = DIMENSION*DIMENSION+6;	// Avoid large arrays in dialog boxes
}

void DialogBuilder::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	int nVar, nArray;

	for (nVar = 0; nVar < m_integer.size(); nVar ++)
	{
		DDX_Text(pDX, id_integer[nVar], m_integer[nVar]);
	}
	for (nVar = 0; nVar < m_double.size(); nVar ++)
	{
		DDX_Text(pDX, id_double[nVar], m_double[nVar]);
	}
	// XXX Integer array
	for (nArray = 0; nArray < m_double_array.size(); nArray ++)
	{
		for (int nField = 0; nField < m_double_array[nArray].size(); nField++)
		{
			DDX_Text(pDX, id_double_array[nArray][nField], m_double_array[nArray][nField]);
		}
	}
}


BEGIN_MESSAGE_MAP(DialogBuilder, CDialog)
// Don't know if we need this?
END_MESSAGE_MAP()


int DialogBuilder::DoModal_DM(Introspective *pIntrospective)
{
	MetaObjectProtocol *mop = MetaObjectProtocol::Instance();
	ClassData * pData = mop->GetData(pIntrospective->ClassID());

	// Name of class
	std::string className = pIntrospective->GetName();
	dialog_template_packed.SetCaption(_T(className.c_str()));

	// Data fields: how many?
	ClassData::field_vector &fieldVector = pData->GetFields();
	int nFields = fieldVector.size();

	// Be extravagant: reserve room for storage of each type based on total number in dialog box

	// Clear storage for values and ID's
	m_integer.clear();
	id_integer.clear();
	m_double.clear();
	id_double.clear();

	// Arrays XXX
	int idd_current_user_number;
	int nVar;	// Stupid Microsoft problem
	for (nVar = 0; nVar < nFields; nVar ++)
	{
		// Local pointer for field vector, ExtTypeInfo
		DataField *df = fieldVector[nVar];
		ExtTypeInfo *eti = fieldVector[nVar]->GetExtTypeInfo();

		// Static text box for name
		std::string varName = df->GetName();
		int length = varName.length()*4+8;	// Dialog-box coordinates: 4 per char width, 8 per char height
		dialog_template_packed.AddControl(0,0,length,10,IDC_STATIC,WS_VISIBLE|WS_CHILD,
					_T("Static"),_T(varName.c_str()));

		// Style of edit box
		UINT style;
		if (df->IsMutable())
			style = WS_VISIBLE|WS_CHILD|WS_TABSTOP|WS_BORDER;
		else
			style = WS_VISIBLE|WS_CHILD|WS_TABSTOP|WS_BORDER|WS_DISABLED;

		if (!df->IsArray()) // Single variable
		{
			idd_current_user_number = DialogIDManager::Instance()->GetID();
			if (eti->isInt()) {
				m_integer.push_back(df->GetInt(pIntrospective));
				id_integer.push_back(idd_current_user_number);
			} else if (eti->isDouble()) {
				m_double.push_back(df->GetDouble(pIntrospective));
				id_double.push_back(idd_current_user_number);
			}
			dialog_template_packed.AddControl(0,40,80,12,idd_current_user_number,style,
				_T("Edit"),_T(""));	// XXX I don't know why I need a caption for an edit control
//			idd_current_user_number++;
		}
		else	// Array
		{
			int iDim = df->GetNumberOfDimensions(pIntrospective);
			int iLen = df->GetDimension(pIntrospective,0);
			if (iDim==1 && iLen < maxNumberOfFieldsPerArray)	// Only 1-D arrays for now
			{
				if (eti->isDouble()) 
				{
					const double * dp = df->GetPDouble(pIntrospective);
					int nArrays = m_double_array.size()+1;
					m_double_array.resize(nArrays);	// Add one more array
					id_double_array.resize(nArrays);	// Add one more array
					for (int i=0; i<iLen; i++) 
					{
						idd_current_user_number = DialogIDManager::Instance()->GetID();
						m_double_array[nArrays-1].push_back(dp[i]);
						id_double_array[nArrays-1].push_back(idd_current_user_number);
						dialog_template_packed.AddControl(0,40,80,12,idd_current_user_number,style,
							_T("Edit"),_T(""));	// XXX I don't know why I need a caption for an edit control
//						idd_current_user_number++;
					}
				}
			}			
		}
	}
	
// Call each time it's used just before DoModal_DM(). 

	dialog_template_packed.AddControl(0,140,30,16,IDOK,WS_VISIBLE|WS_CHILD|WS_TABSTOP,
							_T("Button"),_T("OK"));
	dialog_template_packed.AddControl(0,140,30,16,IDCANCEL,WS_VISIBLE|WS_CHILD|WS_TABSTOP,
							_T("Button"),_T("Cancel"));


	InitModalIndirect(dialog_template_packed.LockTemplate());
	int doModalReturn = DoModal();
	dialog_template_packed.UnlockTemplate();

	if (doModalReturn == IDOK)
	{
		int ret = simThread->Pause();
		bool wasRunning = (ret!=Thread::NO_THREAD);
		bool wasPaused = (ret==Thread::ALREADY_PAUSED);

		// Keep track of where we are on the various lists
		int iIntegers=0, iDoubles=0, iIntegerArrays=0, iDoubleArrays=0;
	
		for (nVar = 0; nVar < nFields; nVar ++)
		{
			// Local pointer for field vector, ExtTypeInfo
			DataField *df = fieldVector[nVar];
			if (df->IsMutable())
			{
				ExtTypeInfo *eti = df->GetExtTypeInfo();	
				if (!df->IsArray()) 
				{
					if (eti->isInt()) 
					{
						if (m_integer[iIntegers] != df->GetInt(pIntrospective))
							df->SetInt(pIntrospective,m_integer[iIntegers]);
						iIntegers++;
					} 
					else if (eti->isDouble()) 
					{
						// Dialog box changes numbers in 14th decimal place even when no change
						double smallDifference = 1.0e-14;	// We'll forbid very small changes: 
						double newV = df->GetDouble(pIntrospective);
						double oldV = m_double[iDoubles];
						double magnitude = fabs(newV) + fabs(oldV);
						if ((newV-oldV != 0) 
							&& (fabs(newV - oldV)/magnitude > smallDifference))
						{
							df->SetDouble(pIntrospective,m_double[iDoubles]);
						}
						iDoubles++;
					}
				}
				else // Array
				{
					int iDim = df->GetNumberOfDimensions(pIntrospective);
					int iLen = df->GetDimension(pIntrospective,0);
					if (iDim==1 && iLen < maxNumberOfFieldsPerArray)	// Only 1-D arrays for now
					{
						if (eti->isDouble()) 
						{
							df->SetPDouble(pIntrospective, &(m_double_array[iDoubleArrays][0]));
							const double * dp2 = df->GetPDouble(pIntrospective);
							iDoubleArrays++;
						}
					}
				}
			}
		}
		if (wasRunning && !wasPaused) {
			int ret = simThread->Resume();	// Resume thread
			if (ret!=Thread::SUCCESS) {
				int i = 1; //XXX debug point.
			}
		}

	}
	
	ReleaseIDs(id_double);
	ReleaseIDs(id_integer);
	int jj;
	for (jj=0; jj<id_double_array.size(); jj++)
		ReleaseIDs(id_double_array[jj]);
	for (jj=0; jj<id_integer_array.size(); jj++)
		ReleaseIDs(id_integer_array[jj]);

	return doModalReturn;
}

void DialogBuilder::ReleaseIDs(std::vector<UINT>& vec)
{
	DialogIDManager *manager = DialogIDManager::Instance();
	std::vector<UINT>::const_iterator begin = vec.begin();
	std::vector<UINT>::const_iterator end = vec.end();
	while (begin != end) {
		manager->ReleaseID(*begin);
		begin++;
	}
}
