// simThread.cpp

/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

// Main loop: sets up and controls the simulation
// The software is designed so that this can be done in an interpreted, scripting language

#include "stdafx.h"
#include "MD.h"

#include "MDDoc.h"
#include "MDView.h"


UINT simThread( LPVOID pParam)
{
	// This is thread stuff: have to unpack everything from the "other side"
	// forced through the small hole of the pointer
	// Get pointer to view from other thread
	
	// Unpacking HWndStore
	
	HWndStore* phWndStore = (HWndStore *) pParam;
	
    if (phWndStore == NULL ||
	        !phWndStore->IsKindOf(RUNTIME_CLASS(HWndStore)))
		return 1;   // if pObject is not valid
		
	HWND hwnd = phWndStore->hwnd;
	CMDView *atomView = phWndStore->pcmdView;
	atomView->Detach();	// Detach former view
	atomView->Attach(hwnd);
	CMDDoc *pDoc = phWndStore->pcmdDoc;
	MDSimulation* sim = phWndStore->pSim;
	
	// Animation
	sim->GetListOfAtoms()->Attach(atomView);
	atomView->SetDiameter(sim->GetDiameter());    // returns the size of an atom
	atomView->SetPlottedRegionSize(sim->GetMaxSize());
	atomView->SetCenter(sim->GetAtomsInitializer()->GetCenter());

	// Create cluster
//	sim->GetAtomsInitializer()->Create(sim->GetListOfAtoms());	// Sets up cluster

/*	// XXX For class, should change these
//	atomView->SetPlottedRegionSize(3*sim->GetMaxSize());
//	atomView->SetCenter(sim->GetAtomsInitializer()->GetCenter());
	double plotCenter[2] = {0,0};
	atomView->SetCenter(plotCenter);
	atomView->SetPlottedRegionSize(40);

	// Change for class: old
	// Create cluster
// sim->GetAtomsInitializer()->Create(sim->GetListOfAtoms());	// Sets up cluster

// Thought for class
	PrimitiveListOfAtoms *atoms0 = (PrimitiveListOfAtoms *)sim->listOfListsOfAtoms.components[0];
	PrimitiveListOfAtoms *atoms1 = (PrimitiveListOfAtoms *)sim->listOfListsOfAtoms.components[1];
	RectangularClusterInitializer* rect = (RectangularClusterInitializer *)sim->listOfAtomsInitializers.components[0];
	SphericalClusterInitializer* sphere = (SphericalClusterInitializer *)sim->listOfAtomsInitializers.components[1];
	

	rect->Create(atoms0);	// Sets up surface

*/

	sim->GetListOfAtoms()->Notify();

	bool stopNow = false;
	while (!pDoc->simThreadController.StopNow())
	{
		sim->GetAtomsMover()->Move(sim->GetListOfAtoms());
		sim->GetListOfAtoms()->Notify();

		Sleep(0);	// Improves responsiveness

		if (pDoc->simThreadController.PauseNow())
			pDoc->simThreadController.Paused();	// Announce paused: wait for resume
	}
	
	pDoc->simThreadController.Stopped();
	return 1;
}
